# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""services helper functions."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import encoding
from apitools.base.py import exceptions as apitools_exceptions
from apitools.base.py import list_pager
from googlecloudsdk.api_lib.services import exceptions
from googlecloudsdk.api_lib.util import apis_internal
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core import transport
from googlecloudsdk.core.credentials import transports

_PROJECT_RESOURCE = 'projects/%s'
_PROJECT_SERVICE_RESOURCE = 'projects/%s/services/%s'
_CONSUMER_SERVICE_RESOURCE = '%s/services/%s'
_LIMIT_OVERRIDE_RESOURCE = '%s/consumerOverrides/%s'
_VALID_CONSUMER_PREFIX = {'projects/', 'folders/', 'organizations/'}
_V1_VERSION = 'v1'
_V1BETA1_VERSION = 'v1beta1'
_V1ALPHA_VERSION = 'v1alpha'

# Map of services which should be protected from being disabled by
# prompting the user for  confirmation
_PROTECTED_SERVICES = {
    'anthos.googleapis.com': ('Warning: Disabling this service will '
                              'also automatically disable any running '
                              'Anthos clusters.')
}


def GetProtectedServiceWarning(service_name):
  """Return the warning message associated with a protected service."""
  return _PROTECTED_SERVICES.get(service_name)


def EnableApiCall(project, service):
  """Make API call to enable a specific service.

  Args:
    project: The project for which to enable the service.
    service: The identifier of the service to enable, for example
      'serviceusage.googleapis.com'.

  Raises:
    exceptions.EnableServicePermissionDeniedException: when enabling API fails.
    apitools_exceptions.HttpError: Another miscellaneous error with the service.

  Returns:
    The result of the operation
  """
  client = _GetClientInstance()
  messages = client.MESSAGES_MODULE

  request = messages.ServiceusageServicesEnableRequest(
      name=_PROJECT_SERVICE_RESOURCE % (project, service))
  try:
    return client.services.Enable(request)
  except (apitools_exceptions.HttpForbiddenError,
          apitools_exceptions.HttpNotFoundError) as e:
    exceptions.ReraiseError(e,
                            exceptions.EnableServicePermissionDeniedException)


def BatchEnableApiCall(project, services):
  """Make API call to batch enable services.

  Args:
    project: The project for which to enable the services.
    services: Iterable of identifiers of services to enable.

  Raises:
    exceptions.EnableServicePermissionDeniedException: when enabling API fails.
    apitools_exceptions.HttpError: Another miscellaneous error with the service.

  Returns:
    The result of the operation
  """
  client = _GetClientInstance()
  messages = client.MESSAGES_MODULE

  request = messages.ServiceusageServicesBatchEnableRequest(
      batchEnableServicesRequest=messages.BatchEnableServicesRequest(
          serviceIds=services),
      parent=_PROJECT_RESOURCE % project)
  try:
    return client.services.BatchEnable(request)
  except (apitools_exceptions.HttpForbiddenError,
          apitools_exceptions.HttpNotFoundError) as e:
    exceptions.ReraiseError(e,
                            exceptions.EnableServicePermissionDeniedException)


def DisableApiCall(project, service, force=False):
  """Make API call to disable a specific service.

  Args:
    project: The project for which to enable the service.
    service: The identifier of the service to disable, for example
      'serviceusage.googleapis.com'.
    force: disable the service even if there are enabled services which depend
      on it. This also disables the services which depend on the service to be
      disabled.

  Raises:
    exceptions.EnableServicePermissionDeniedException: when disabling API fails.
    apitools_exceptions.HttpError: Another miscellaneous error with the service.

  Returns:
    The result of the operation
  """
  client = _GetClientInstance()
  messages = client.MESSAGES_MODULE

  check = messages.DisableServiceRequest.CheckIfServiceHasUsageValueValuesEnum.CHECK
  if force:
    check = messages.DisableServiceRequest.CheckIfServiceHasUsageValueValuesEnum.SKIP
  request = messages.ServiceusageServicesDisableRequest(
      name=_PROJECT_SERVICE_RESOURCE % (project, service),
      disableServiceRequest=messages.DisableServiceRequest(
          disableDependentServices=force,
          checkIfServiceHasUsage=check,
      ),
  )
  try:
    return client.services.Disable(request)
  except (apitools_exceptions.HttpForbiddenError,
          apitools_exceptions.HttpNotFoundError) as e:
    exceptions.ReraiseError(e,
                            exceptions.EnableServicePermissionDeniedException)
  except apitools_exceptions.HttpBadRequestError as e:
    log.status.Print('Provide the --force flag if you wish to force disable '
                     'services.')
    exceptions.ReraiseError(e, exceptions.Error)


def GetService(project, service):
  """Get a service.

  Args:
    project: The project for which to get the service.
    service: The service to get.

  Raises:
    exceptions.GetServicePermissionDeniedException: when getting service fails.
    apitools_exceptions.HttpError: Another miscellaneous error with the service.

  Returns:
    The service configuration.
  """
  client = _GetClientInstance()
  messages = client.MESSAGES_MODULE

  request = messages.ServiceusageServicesGetRequest(
      name=_PROJECT_SERVICE_RESOURCE % (project, service))
  try:
    return client.services.Get(request)
  except (apitools_exceptions.HttpForbiddenError,
          apitools_exceptions.HttpNotFoundError) as e:
    exceptions.ReraiseError(e, exceptions.GetServicePermissionDeniedException)


def IsServiceEnabled(service):
  client = _GetClientInstance()
  messages = client.MESSAGES_MODULE
  return service.state == messages.GoogleApiServiceusageV1Service.StateValueValuesEnum.ENABLED


def ListServices(project, enabled, page_size, limit):
  """Make API call to list services.

  Args:
    project: The project for which to list services.
    enabled: List only enabled services.
    page_size: The page size to list.
    limit: The max number of services to display.

  Raises:
    exceptions.ListServicesPermissionDeniedException: when listing services
    fails.
    apitools_exceptions.HttpError: Another miscellaneous error with the service.

  Returns:
    The list of services
  """
  client = _GetClientInstance()
  messages = client.MESSAGES_MODULE

  if enabled:
    service_filter = 'state:ENABLED'
  else:
    service_filter = None
  request = messages.ServiceusageServicesListRequest(
      filter=service_filter, parent=_PROJECT_RESOURCE % project)
  try:
    return list_pager.YieldFromList(
        client.services,
        request,
        limit=limit,
        batch_size_attribute='pageSize',
        batch_size=page_size,
        field='services')
  except (apitools_exceptions.HttpForbiddenError,
          apitools_exceptions.HttpNotFoundError) as e:
    exceptions.ReraiseError(e,
                            exceptions.EnableServicePermissionDeniedException)


def GetOperation(name):
  """Make API call to get an operation.

  Args:
    name: The name of operation.

  Raises:
    exceptions.OperationErrorException: when the getting operation API fails.
    apitools_exceptions.HttpError: Another miscellaneous error with the service.

  Returns:
    The result of the operation
  """
  client = _GetClientInstance()
  messages = client.MESSAGES_MODULE
  request = messages.ServiceusageOperationsGetRequest(name=name)
  try:
    return client.operations.Get(request)
  except (apitools_exceptions.HttpForbiddenError,
          apitools_exceptions.HttpNotFoundError) as e:
    exceptions.ReraiseError(e, exceptions.OperationErrorException)


def GenerateServiceIdentity(project, service):
  """Generate a service identity.

  Args:
    project: The project to generate a service identity for.
    service: The service to generate a service identity for.

  Raises:
    exceptions.GenerateServiceIdentityPermissionDeniedException: when generating
    service identity fails.
    apitools_exceptions.HttpError: Another miscellaneous error with the service.

  Returns:
    A dict with the email and uniqueId of the generated service identity. If
    service does not have a default identity, the response will be an empty
    dictionary.
  """
  client = _GetClientInstance(version=_V1BETA1_VERSION)
  messages = client.MESSAGES_MODULE

  request = messages.ServiceusageServicesGenerateServiceIdentityRequest(
      parent=_PROJECT_SERVICE_RESOURCE % (project, service))
  try:
    op = client.services.GenerateServiceIdentity(request)
    response = encoding.MessageToDict(op.response)
    # Only keep email and uniqueId from the response.
    # If the response doesn't contain these keys, the returned dictionary will
    # not contain them either.
    return {k: response[k] for k in ('email', 'uniqueId') if k in response}
  except (apitools_exceptions.HttpForbiddenError,
          apitools_exceptions.HttpNotFoundError) as e:
    exceptions.ReraiseError(
        e, exceptions.GenerateServiceIdentityPermissionDeniedException)


def ListQuotaMetrics(consumer, service, page_size=None, limit=None):
  """List service quota metrics for a consumer.

  Args:
    consumer: The consumer to list metrics for, e.g. "projects/123".
    service: The service to list metrics for.
    page_size: The page size to list.
    limit: The max number of metrics to return.

  Raises:
    exceptions.PermissionDeniedException: when listing metrics fails.
    apitools_exceptions.HttpError: Another miscellaneous error with the service.

  Returns:
    The list of quota metrics
  """
  _ValidateConsumer(consumer)
  client = _GetClientInstance(version=_V1BETA1_VERSION)
  messages = client.MESSAGES_MODULE

  request = messages.ServiceusageServicesConsumerQuotaMetricsListRequest(
      parent=_CONSUMER_SERVICE_RESOURCE % (consumer, service))
  return list_pager.YieldFromList(
      client.services_consumerQuotaMetrics,
      request,
      limit=limit,
      batch_size_attribute='pageSize',
      batch_size=page_size,
      field='metrics')


def UpdateQuotaOverrideCall(consumer,
                            service,
                            metric,
                            unit,
                            dimensions,
                            value,
                            force=False):
  """Update a quota override.

  Args:
    consumer: The consumer to update a quota override for, e.g. "projects/123".
    service: The service to update a quota override for.
    metric: The quota metric name.
    unit: The unit of quota metric.
    dimensions: The dimensions of the override in dictionary format. It can be
      None.
    value: The override integer value.
    force: Force override update even if the change results in a substantial
      decrease in available quota.

  Raises:
    exceptions.UpdateQuotaOverridePermissionDeniedException: when updating an
    override fails.
    apitools_exceptions.HttpError: Another miscellaneous error with the service.

  Returns:
    The quota override operation.
  """
  _ValidateConsumer(consumer)
  client = _GetClientInstance(version=_V1BETA1_VERSION)
  messages = client.MESSAGES_MODULE

  dimensions_message = _GetDimensions(messages, dimensions)
  request = messages.ServiceusageServicesConsumerQuotaMetricsImportConsumerOverridesRequest(
      parent=_CONSUMER_SERVICE_RESOURCE % (consumer, service),
      importConsumerOverridesRequest=messages.ImportConsumerOverridesRequest(
          inlineSource=messages.OverrideInlineSource(
              overrides=[
                  messages.QuotaOverride(
                      metric=metric,
                      unit=unit,
                      overrideValue=value,
                      dimensions=dimensions_message)
              ],),
          force=force),
  )
  try:
    return client.services_consumerQuotaMetrics.ImportConsumerOverrides(request)
  except (apitools_exceptions.HttpForbiddenError,
          apitools_exceptions.HttpNotFoundError) as e:
    exceptions.ReraiseError(
        e, exceptions.UpdateQuotaOverridePermissionDeniedException)


def DeleteQuotaOverrideCall(consumer,
                            service,
                            metric,
                            unit,
                            override_id,
                            force=False):
  """Delete a quota override.

  Args:
    consumer: The consumer to delete a quota override for, e.g. "projects/123".
    service: The service to delete a quota aoverride for.
    metric: The quota metric name.
    unit: The unit of quota metric.
    override_id: The override ID.
    force: Force override deletion even if the change results in a substantial
      decrease in available quota.

  Raises:
    exceptions.DeleteQuotaOverridePermissionDeniedException: when deleting an
    override fails.
    apitools_exceptions.HttpError: Another miscellaneous error with the service.

  Returns:
    The quota override operation.
  """
  _ValidateConsumer(consumer)
  client = _GetClientInstance(version=_V1BETA1_VERSION)
  messages = client.MESSAGES_MODULE

  parent = _GetMetricResourceName(consumer, service, metric, unit)
  name = _LIMIT_OVERRIDE_RESOURCE % (parent, override_id)
  request = messages.ServiceusageServicesConsumerQuotaMetricsLimitsConsumerOverridesDeleteRequest(
      name=name,
      force=force,
  )
  try:
    return client.services_consumerQuotaMetrics_limits_consumerOverrides.Delete(
        request)
  except (apitools_exceptions.HttpForbiddenError,
          apitools_exceptions.HttpNotFoundError) as e:
    exceptions.ReraiseError(
        e, exceptions.DeleteQuotaOverridePermissionDeniedException)


def _GetDimensions(messages, dimensions):
  if dimensions is None:
    return None
  dt = messages.QuotaOverride.DimensionsValue
  # sorted by key strings to maintain the unit test behavior consistency.
  return dt(
      additionalProperties=[
          dt.AdditionalProperty(key=k, value=dimensions[k])
          for k in sorted(dimensions.keys())
      ],)


def _GetMetricResourceName(consumer, service, metric, unit):
  """Get the metric resource name from metric name and unit.

  Args:
    consumer: The consumer to manage an override for, e.g. "projects/123".
    service: The service to manage an override for.
    metric: The quota metric name.
    unit: The unit of quota metric.

  Raises:
    exceptions.Error: when the limit with given metric and unit is not found.

  Returns:
    The quota override operation.
  """
  metrics = ListQuotaMetrics(consumer, service)
  for m in metrics:
    if m.metric == metric:
      for q in m.consumerQuotaLimits:
        if q.unit == unit:
          return q.name
  raise exceptions.Error('limit not found with name "%s" and unit "%s".' %
                         (metric, unit))


def _ValidateConsumer(consumer):
  for prefix in _VALID_CONSUMER_PREFIX:
    if consumer.startswith(prefix):
      return
  raise exceptions.Error('invalid consumer format "%s".' % consumer)


def _GetClientInstance(version='v1'):
  """Get a client instance for service usage."""
  # pylint:disable=protected-access
  # Specifically disable resource quota in all cases for service management.
  # We need to use this API to turn on APIs and sometimes the user doesn't have
  # this API turned on. We should always use the shared project to do this
  # so we can bootstrap users getting the appropriate APIs enabled. If the user
  # has explicitly set the quota project, then respect that.
  enable_resource_quota = (
      properties.VALUES.billing.quota_project.IsExplicitlySet())
  http_client = transports.GetApitoolsTransport(
      response_encoding=transport.ENCODING,
      enable_resource_quota=enable_resource_quota)
  return apis_internal._GetClientInstance(
      'serviceusage', version, http_client=http_client)



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""API helpers for interacting with attestations."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import encoding
from googlecloudsdk.api_lib.container.binauthz import apis


class Client(object):
  """A client for interacting with attestations."""

  def __init__(self, api_version=None):
    self.client = apis.GetClientInstance(api_version)
    self.messages = apis.GetMessagesModule(api_version)
    self.api_version = api_version

  def Test(self, occurrence, attestor_ref):
    """Validates a v1 Occurrence using the TestAttestationOccurrence RPC endpoint."""
    # The AttestationOccurrence being passed in is an instance of the message
    # type from the Container Analysis API but the
    # TestAttestationOccurrenceRequest method generated in the Binary
    # Authorization API requires the Binauthz generated version of the type.
    binauthz_attestation = encoding.JsonToMessage(
        self.messages.AttestationOccurrence,
        encoding.MessageToJson(occurrence.attestation))
    test_attestation_request = self.messages.ValidateAttestationOccurrenceRequest(
        attestation=binauthz_attestation,
        occurrenceNote=occurrence.noteName,
        occurrenceResourceUri=occurrence.resourceUri,
    )
    validation_request = self.messages.BinaryauthorizationProjectsAttestorsValidateAttestationOccurrenceRequest(
        attestor=attestor_ref.RelativeName(),
        validateAttestationOccurrenceRequest=test_attestation_request)
    validation_response = self.client.projects_attestors.ValidateAttestationOccurrence(
        validation_request)

    return validation_response



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Allows you to write surfaces in terms of logical Eventflow operations."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import contextlib
import datetime
import functools
import random

from apitools.base.py import exceptions as api_exceptions
from googlecloudsdk.api_lib.events import custom_resource_definition
from googlecloudsdk.api_lib.events import iam_util
from googlecloudsdk.api_lib.events import metric_names
from googlecloudsdk.api_lib.events import source
from googlecloudsdk.api_lib.events import trigger
from googlecloudsdk.api_lib.run import secret
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.api_lib.util import apis_internal
from googlecloudsdk.command_lib.events import anthosevents_operations
from googlecloudsdk.command_lib.events import exceptions
from googlecloudsdk.command_lib.events import stages
from googlecloudsdk.command_lib.events import util
from googlecloudsdk.command_lib.run import serverless_operations
from googlecloudsdk.command_lib.util.apis import arg_utils
from googlecloudsdk.command_lib.util.apis import registry
from googlecloudsdk.core import metrics
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


_EVENT_SOURCES_LABEL_SELECTOR = 'duck.knative.dev/source=true'

_METADATA_LABELS_FIELD = 'metadata.labels'

_SERVICE_ACCOUNT_KEY_COLLECTION = 'iam.projects.serviceAccounts.keys'

_CORE_CLIENT_VERSION = 'v1'
_CRD_CLIENT_VERSION = 'v1beta1'


@contextlib.contextmanager
def Connect(conn_context):
  """Provide a EventflowOperations instance to use.

  If we're using the GKE Serverless Add-on, connect to the relevant cluster.
  Otherwise, connect to the right region of GSE.

  Arguments:
    conn_context: a context manager that yields a ConnectionInfo and manages a
      dynamic context that makes connecting to serverless possible.

  Yields:
    A EventflowOperations instance.
  """
  with conn_context as conn_info:
    if conn_info.api_name == 'anthosevents':
      yield anthosevents_operations.Connect(conn_info)
    else:
      yield ConnectEventManaged(conn_info)


def ConnectEventManaged(conn_context):
  """Provides an EventflowOperations instance to use, for managed events."""

  # pylint: disable=protected-access
  client = apis_internal._GetClientInstance(
      conn_context.api_name,
      conn_context.api_version,
      # Only check response if not connecting to GKE
      check_response_func=apis.CheckResponseForApiEnablement()
      if conn_context.supports_one_platform else None,
      http_client=conn_context.HttpClient())
  # This client is used for working with core resources (e.g. Secrets) in
  # Cloud Run for Anthos.
  core_client = None
  if not conn_context.supports_one_platform:
    core_client = apis_internal._GetClientInstance(
        conn_context.api_name,
        _CORE_CLIENT_VERSION,
        http_client=conn_context.HttpClient())
  # This client is only used to get CRDs because the api group they are
  # under uses different versioning in k8s
  crd_client = apis_internal._GetClientInstance(
      conn_context.api_name,
      _CRD_CLIENT_VERSION,
      http_client=conn_context.HttpClient())
  # pylint: enable=protected-access
  return EventflowOperations(client, conn_context.api_version,
                             conn_context.region, core_client, crd_client)

# TODO(b/149793348): Remove this grace period
_POLLING_GRACE_PERIOD = datetime.timedelta(seconds=15)


class TimeLockedUnfailingConditionPoller(serverless_operations.ConditionPoller):
  """Condition poller that never fails and is only done on success for a period of time.

  Knative Eventing occasionally returns Ready == False on a resource that will
  shortly become Ready == True. In these cases, we cannot rely upon that False
  status as an indication of a terminal failure. Instead, only Ready == True can
  be relied upon as a terminal state and all other statuses (False, Unknown)
  simply mean not currently successful, but provide no indication if this is a
  temporary or permanent state.

  This condition poller never fails a stage for that reason, and therefore is
  never done until successful.

  This behavior only exists for a period of time, after which it acts like a
  normal condition poller.
  """

  def __init__(self,
               getter,
               tracker,
               dependencies=None,
               grace_period=_POLLING_GRACE_PERIOD):
    super(TimeLockedUnfailingConditionPoller,
          self).__init__(getter, tracker, dependencies)
    self._grace_period = _POLLING_GRACE_PERIOD
    self._start_time = datetime.datetime.now()

  def _HasGracePeriodPassed(self):
    return datetime.datetime.now() - self._start_time > self._grace_period

  def IsDone(self, conditions):
    """Within grace period -  this only checks for IsReady rather than IsTerminal.

    Args:
      conditions: A condition.Conditions object.

    Returns:
      A bool indicating whether `conditions` is ready.
    """
    if self._HasGracePeriodPassed():
      return super(TimeLockedUnfailingConditionPoller, self).IsDone(conditions)

    if conditions is None:
      return False
    return conditions.IsReady()

  def Poll(self, unused_ref):
    """Within grace period - this polls like normal but does not raise on failure.

    Args:
      unused_ref: A string representing the operation reference. Unused and may
        be None.

    Returns:
      A condition.Conditions object or None if there's no conditions on the
        resource or if the conditions are not fresh (the generation on the
        resource doesn't match the observedGeneration)
    """
    if self._HasGracePeriodPassed():
      return super(TimeLockedUnfailingConditionPoller, self).Poll(unused_ref)

    conditions = self.GetConditions()

    if conditions is None or not conditions.IsFresh():
      return None

    conditions_message = conditions.DescriptiveMessage()
    if conditions_message:
      self._tracker.UpdateHeaderMessage(conditions_message)

    self._PollTerminalSubconditions(conditions, conditions_message)

    if conditions.IsReady():
      self._tracker.UpdateHeaderMessage(self._ready_message)
      self._tracker.Tick()

    return conditions

  def _PossiblyFailStage(self, condition, message):
    """Within grace period - stages are never marked as failed."""
    if self._HasGracePeriodPassed():
      # pylint:disable=protected-access
      return super(TimeLockedUnfailingConditionPoller,
                   self)._PossiblyFailStage(condition, message)


class TriggerConditionPoller(TimeLockedUnfailingConditionPoller):
  """A ConditionPoller for triggers."""


class SourceConditionPoller(TimeLockedUnfailingConditionPoller):
  """A ConditionPoller for sources."""


class EventflowOperations(object):
  """Client used by Eventflow to communicate with the actual API."""

  def __init__(self, client, api_version, region, core_client, crd_client):
    """Inits EventflowOperations with given API clients.

    Args:
      client: The API client for interacting with Kubernetes Cloud Run APIs.
      api_version: Version of resources & clients (v1alpha1, v1beta1)
      region: str, The region of the control plane if operating against
        hosted Cloud Run, else None.
      core_client: The API client for queries against core resources if
        operating against Cloud Run for Anthos, else None.
      crd_client: The API client for querying for CRDs
    """
    self._client = client
    self._api_version = api_version
    self._core_client = core_client
    self._crd_client = crd_client
    self._region = region

  def IsCluster(self):
    return False

  @property
  def client(self):
    return self._client

  @property
  def messages(self):
    return self._client.MESSAGES_MODULE

  def GetTrigger(self, trigger_ref):
    """Returns the referenced trigger."""
    request = self.messages.RunNamespacesTriggersGetRequest(
        name=trigger_ref.RelativeName())
    try:
      with metrics.RecordDuration(metric_names.GET_TRIGGER):
        response = self._client.namespaces_triggers.Get(request)
    except api_exceptions.HttpNotFoundError:
      return None
    return trigger.Trigger(response, self.messages)

  def CreateTrigger(self, trigger_ref, source_obj, event_type, trigger_filters,
                    target_service, broker):
    """Create a trigger that sends events to the target service.

    Args:
      trigger_ref: googlecloudsdk.core.resources.Resource, trigger resource.
      source_obj: source.Source. The source object to be created after the
        trigger. If creating a custom event, this may be None.
      event_type: str, the event type the source will filter by.
      trigger_filters: collections.OrderedDict()
      target_service: str, name of the Cloud Run service to subscribe.
      broker: str, name of the broker to act as a sink for the source.

    Returns:
      trigger.Trigger of the created trigger.
    """
    trigger_obj = trigger.Trigger.New(self._client, trigger_ref.Parent().Name())
    trigger_obj.name = trigger_ref.Name()
    if source_obj is not None:
      trigger_obj.dependency = source_obj
      # TODO(b/141617597): Set to str(random.random()) without prepended string
      trigger_obj.filter_attributes[
          trigger.SOURCE_TRIGGER_LINK_FIELD] = 'link{}'.format(random.random())
    trigger_obj.filter_attributes[trigger.EVENT_TYPE_FIELD] = event_type

    # event/flags.py ensures filter key doesn't include disallowed fields
    trigger_obj.filter_attributes.update(trigger_filters)

    trigger_obj.subscriber = target_service
    trigger_obj.broker = broker

    request = self.messages.RunNamespacesTriggersCreateRequest(
        trigger=trigger_obj.Message(),
        parent=trigger_ref.Parent().RelativeName())
    try:
      with metrics.RecordDuration(metric_names.CREATE_TRIGGER):
        response = self._client.namespaces_triggers.Create(request)
    except api_exceptions.HttpConflictError:
      raise exceptions.TriggerCreationError(
          'Trigger [{}] already exists.'.format(trigger_obj.name))

    return trigger.Trigger(response, self.messages)

  def PollTrigger(self, trigger_ref, tracker):
    """Wait for trigger to be Ready == True."""
    trigger_getter = functools.partial(self.GetTrigger, trigger_ref)
    poller = TriggerConditionPoller(trigger_getter, tracker,
                                    stages.TriggerSourceDependencies())
    util.WaitForCondition(poller, exceptions.TriggerCreationError)

  def ListTriggers(self, namespace_ref):
    """Returns a list of existing triggers in the given namespace."""
    request = self.messages.RunNamespacesTriggersListRequest(
        parent=namespace_ref.RelativeName())
    with metrics.RecordDuration(metric_names.LIST_TRIGGERS):
      response = self._client.namespaces_triggers.List(request)
    return [trigger.Trigger(item, self.messages) for item in response.items]

  def DeleteTrigger(self, trigger_ref):
    """Deletes the referenced trigger."""
    request = self.messages.RunNamespacesTriggersDeleteRequest(
        name=trigger_ref.RelativeName())
    try:
      with metrics.RecordDuration(metric_names.DELETE_TRIGGER):
        self._client.namespaces_triggers.Delete(request)
    except api_exceptions.HttpNotFoundError:
      raise exceptions.TriggerNotFound(
          'Trigger [{}] not found.'.format(trigger_ref.Name()))

  def _FindSourceMethod(self, source_crd, method_name):
    """Returns the given method for the given source kind.

    Because every source has its own methods for rpc requests, this helper is
    used to get the underlying methods for a request against a given source
    type. Preferred usage of this private message is via the public
    methods: self.Source{Method_name}Method.

    Args:
      source_crd: custom_resource_definition.SourceCustomResourceDefinition,
        source CRD of the type we want to make a request against.
      method_name: str, the method name (e.g. "get", "create", "list", etc.)

    Returns:
      registry.APIMethod, holds information for the requested method.
    """
    return registry.GetMethod(
        util.SOURCE_COLLECTION_NAME.format(
            plural_kind=source_crd.source_kind_plural),
        method_name,
        api_version=self._api_version)

  def SourceGetMethod(self, source_crd):
    """Returns the request method for a Get request of this source."""
    return self._FindSourceMethod(source_crd, 'get')

  def SourceCreateMethod(self, source_crd):
    """Returns the request method for a Create request of this source."""
    return self._FindSourceMethod(source_crd, 'create')

  def SourceDeleteMethod(self, source_crd):
    """Returns the request method for a Delete request of this source."""
    return self._FindSourceMethod(source_crd, 'delete')

  def GetSource(self, source_ref, source_crd):
    """Returns the referenced source."""
    request_method = self.SourceGetMethod(source_crd)
    request_message_type = request_method.GetRequestType()
    request = request_message_type(name=source_ref.RelativeName())
    try:
      with metrics.RecordDuration(metric_names.GET_SOURCE):
        response = request_method.Call(request, client=self._client)
    except api_exceptions.HttpNotFoundError:
      return None
    return source.Source(response, self.messages, source_crd.source_kind)

  def CreateSource(self, source_obj, source_crd, owner_trigger, namespace_ref,
                   broker, parameters):
    """Create an source with the specified event type and owner trigger.

    Args:
      source_obj: source.Source. The source object being created.
      source_crd: custom_resource_definition.SourceCRD, the source crd for the
        source to create
      owner_trigger: trigger.Trigger, trigger to associate as an owner of the
        source.
      namespace_ref: googlecloudsdk.core.resources.Resource, namespace resource.
      broker: str, name of the broker to act as a sink.
      parameters: dict, additional parameters to set on the source spec.

    Returns:
      source.Source of the created source.
    """
    source_obj.ce_overrides[trigger.SOURCE_TRIGGER_LINK_FIELD] = (
        owner_trigger.filter_attributes[trigger.SOURCE_TRIGGER_LINK_FIELD])
    source_obj.owners.append(
        self.messages.OwnerReference(
            apiVersion=owner_trigger.apiVersion,
            kind=owner_trigger.kind,
            name=owner_trigger.name,
            uid=owner_trigger.uid,
            controller=True))
    source_obj.set_sink(broker, self._api_version)
    arg_utils.ParseStaticFieldsIntoMessage(source_obj.spec, parameters)

    request_method = self.SourceCreateMethod(source_crd)
    request_message_type = request_method.GetRequestType()
    request = request_message_type(**{
        request_method.request_field: source_obj.Message(),
        'parent': namespace_ref.RelativeName()})
    try:
      with metrics.RecordDuration(metric_names.CREATE_SOURCE):
        response = request_method.Call(request, client=self._client)
    except api_exceptions.HttpConflictError:
      raise exceptions.SourceCreationError(
          'Source [{}] already exists.'.format(source_obj.name))

    return source.Source(response, self.messages, source_crd.source_kind)

  def PollSource(self, source_obj, event_type, tracker):
    """Wait for source to be Ready == True."""
    source_ref = util.GetSourceRef(source_obj.name, source_obj.namespace,
                                   event_type.crd, False)
    source_getter = functools.partial(
        self.GetSource, source_ref, event_type.crd)
    poller = SourceConditionPoller(source_getter, tracker,
                                   stages.TriggerSourceDependencies())
    util.WaitForCondition(poller, exceptions.SourceCreationError)
    # Manually complete the stage indicating source readiness because we can't
    # track the Ready condition in the ConditionPoller.
    tracker.CompleteStage(stages.SOURCE_READY)

  def DeleteSource(self, source_ref, source_crd):
    """Deletes the referenced source."""
    request_method = self.SourceDeleteMethod(source_crd)
    request_message_type = request_method.GetRequestType()
    request = request_message_type(name=source_ref.RelativeName())
    try:
      with metrics.RecordDuration(metric_names.DELETE_SOURCE):
        request_method.Call(request, client=self._client)
    except api_exceptions.HttpNotFoundError:
      raise exceptions.SourceNotFound(
          '{} events source [{}] not found.'.format(
              source_crd.source_kind, source_ref.Name()))

  def ListSourceCustomResourceDefinitions(self):
    """Returns a list of CRDs for event sources."""
    # Passing the parent field is only needed for hosted, but shouldn't hurt
    # against an actual cluster
    namespace_ref = resources.REGISTRY.Parse(
        properties.VALUES.core.project.Get(),
        collection='run.namespaces',
        api_version=self._api_version)

    messages = self._crd_client.MESSAGES_MODULE
    request = messages.RunCustomresourcedefinitionsListRequest(
        parent=namespace_ref.RelativeName(),
        labelSelector=_EVENT_SOURCES_LABEL_SELECTOR)
    with metrics.RecordDuration(metric_names.LIST_SOURCE_CRDS):
      response = self._crd_client.customresourcedefinitions.List(request)
    source_crds = [
        custom_resource_definition.SourceCustomResourceDefinition(
            item, messages) for item in response.items
    ]
    # Only include CRDs for source kinds that are defined in the api.
    return [s for s in source_crds if hasattr(self.messages, s.source_kind)]

  def UpdateNamespaceWithLabels(self, namespace_ref, labels):
    """Updates an existing namespace with the labels provided.

    If a label already exists, this will replace that label with the value
    provided. This is akin to specifying --overwrite with kubectl.

    Args:
      namespace_ref: googlecloudsdk.core.resources.Resource, namespace resource.
        Note that this should be of the collection "run.api.v1.namespaces" and
        *not* "run.namespaces".
      labels: map[str, str] of label keys and values to patch.

    Returns:
      Namespace that was patched.
    """
    messages = self._core_client.MESSAGES_MODULE
    namespace = messages.Namespace()
    arg_utils.SetFieldInMessage(namespace, _METADATA_LABELS_FIELD, labels)

    old_additional_headers = {}
    try:
      # We need to specify a special content-type for k8s to accept our PATCH.
      # However, this appears to only be settable at the client level, not at
      # the request level. So we'll update the client for our request, and the
      # set it back to the old value afterwards.
      old_additional_headers = self._core_client.additional_http_headers
      additional_headers = old_additional_headers.copy()
      additional_headers['content-type'] = 'application/merge-patch+json'
      self._core_client.additional_http_headers = additional_headers
    except AttributeError:
      # TODO(b/150229881): Remove this try/except block and below.
      # The mocked test client does not have an additional_http_headers attr
      # So we won't be able to test this part.
      pass
    with metrics.RecordDuration(metric_names.UPDATE_NAMESPACE):
      try:
        request = messages.RunApiV1NamespacesPatchRequest(
            name=namespace_ref.RelativeName(),
            namespace=namespace,
            updateMask=_METADATA_LABELS_FIELD)
        response = self._core_client.api_v1_namespaces.Patch(request)
      finally:
        try:
          self._core_client.additional_http_headers = old_additional_headers
        except AttributeError:
          # The mocked test client does not have an additional_http_headers attr
          pass
    return response

  def CreateOrReplaceServiceAccountSecret(self, secret_ref,
                                          service_account_ref):
    """Create a new secret or replace an existing one.

    Secret data contains the key of the given service account.

    Args:
      secret_ref: googlecloudsdk.core.resources.Resource, secret resource.
      service_account_ref: googlecloudsdk.core.resources.Resource, service
        account whose key will be used to create/replace the secret.

    Returns:
      (secret.Secret, googlecloudsdk.core.resources.Resource): tuple of the
        wrapped Secret resource and a ref to the created service account key.
    """
    secret_obj = secret.Secret.New(
        self._core_client, secret_ref.Parent().Name())
    secret_obj.name = secret_ref.Name()
    key = iam_util.CreateServiceAccountKey(service_account_ref)
    secret_obj.data['key.json'] = key.privateKeyData
    key_ref = resources.REGISTRY.ParseResourceId(
        _SERVICE_ACCOUNT_KEY_COLLECTION, key.name, {})

    messages = self._core_client.MESSAGES_MODULE
    with metrics.RecordDuration(metric_names.CREATE_OR_REPLACE_SECRET):
      # Create secret or replace if already exists.
      try:
        request = messages.RunApiV1NamespacesSecretsCreateRequest(
            secret=secret_obj.Message(),
            parent=secret_ref.Parent().RelativeName())
        response = self._core_client.api_v1_namespaces_secrets.Create(request)
      except api_exceptions.HttpConflictError:
        request = messages.RunApiV1NamespacesSecretsReplaceSecretRequest(
            secret=secret_obj.Message(),
            name=secret_ref.RelativeName())
        response = self._core_client.api_v1_namespaces_secrets.ReplaceSecret(
            request)
    return secret.Secret(response, messages), key_ref



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Classes for runtime handling of concept arguments."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.calliope.concepts import util
from googlecloudsdk.core import exceptions
import six


class Error(exceptions.Error):
  """Base class for errors in this module."""


class ParseError(Error):
  """Raised if a concept fails to parse."""

  def __init__(self, presentation_name, message):
    msg = 'Error parsing [{}].\n{}'.format(presentation_name, message)
    super(ParseError, self).__init__(msg)


class RepeatedConceptName(Error):
  """Raised when adding a concept if one with the given name already exists."""

  def __init__(self, concept_name):
    msg = 'Repeated concept name [{}].'.format(concept_name)
    super(RepeatedConceptName, self).__init__(msg)


class RuntimeHandler(object):
  """A handler to hold information about all concept arguments in a command.

  The handler is assigned to 'CONCEPTS' in the argparse namespace and has an
  attribute to match the name of each concept argument in lower snake case.
  """

  def __init__(self):
    # This is set by the ArgumentInterceptor later.
    self.parsed_args = None
    self._arg_name_lookup = {}
    self._all_concepts = []

  def ParsedArgs(self):
    """Basically a lazy property to use during lazy concept parsing."""
    return self.parsed_args

  def AddConcept(self, name, concept_info, required=True):
    """Adds a concept handler for a given concept.

    Args:
      name: str, the name to be used for the presentation spec.
      concept_info: ConceptInfo, the object that holds dependencies of the
        concept.
      required: bool, True if the concept must be parseable, False if not.

    Raises:
      RepeatedConceptName: If the given "name" has already been used with a
        concept.
    """

    class LazyParse(object):
      """Class provided when accessing a concept to lazily parse from args."""

      def __init__(self, parse, arg_getter):
        self.parse = parse
        self.arg_getter = arg_getter

      def Parse(self):
        try:
          return self.parse(self.arg_getter())
        except concepts.InitializationError as e:
          if required:
            raise ParseError(name, six.text_type(e))
          return None

    if hasattr(self, name):
      raise RepeatedConceptName(name)
    setattr(self, name, LazyParse(concept_info.Parse, self.ParsedArgs))
    self._all_concepts.append({
        'name': name,
        'concept_info': concept_info,
        'required': required,
    })
    for _, arg_name in six.iteritems(concept_info.attribute_to_args_map):
      self._arg_name_lookup[util.NormalizeFormat(arg_name)] = concept_info

  def ArgNameToConceptInfo(self, arg_name):
    return self._arg_name_lookup.get(util.NormalizeFormat(arg_name))

  def Reset(self):
    for concept_details in self._all_concepts:
      concept_details['concept_info'].ClearCache()



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Arg parsing and other utilities for Essential Contacts commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import re

from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.util.apis import arg_utils
from googlecloudsdk.core import properties


def GetContactName(args):
  """Returns the full contact name from the args or raises an exception."""
  c = GetContactId(args)
  p = GetParent(args)
  return '{}/contacts/{}'.format(p, c)


def GetContactId(args):
  _ValidateContact(args.CONTACT_ID)
  return args.CONTACT_ID


def _ValidateContact(flag_value):
  if not re.match('^[0-9]+$', flag_value):
    raise exceptions.InvalidArgumentException('contact', flag_value)


def GetParent(args):
  """Returns the parent resource from args or the active gcloud project."""
  if 0 == sum(bool(x) for x in (args.project, args.folder, args.organization)):
    # if neither project, folder, org was specified default to the
    # current project if available.
    args.project = properties.VALUES.core.project.GetOrFail()

  parent = None
  if args.project:
    _ValidateProject(args.project)
    parent = 'projects/%s' % args.project
  elif args.folder:
    _ValidateFolder(args.folder)
    parent = 'folders/%s' % args.folder
  else:
    _ValidateOrganization(args.organization)
    parent = 'organizations/%s' % args.organization

  return parent


def _ValidateProject(flag_value):
  if not re.match('^[a-z0-9-]+$', flag_value):
    raise exceptions.InvalidArgumentException('project', flag_value)


def _ValidateFolder(flag_value):
  if not re.match('^[0-9]+$', flag_value):
    raise exceptions.InvalidArgumentException('folder', flag_value)


def _ValidateOrganization(flag_value):
  if not re.match('^[0-9]+$', flag_value):
    raise exceptions.InvalidArgumentException('organization', flag_value)


def GetNotificationCategories(args, notification_category_enum_message):
  if not args.notification_categories:
    return []
  return [
      arg_utils.ChoiceToEnum(category_choice,
                             notification_category_enum_message)
      for category_choice in args.notification_categories
  ]



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Wrapper for JSON-based Kubernetes object's metadata."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import re

from googlecloudsdk.api_lib.kuberun import kubernetesobject
from googlecloudsdk.api_lib.kuberun import structuredout

AUTHOR_ANNOTATION = 'serving.knative.dev/creator'
INITIAL_SCALE_ANNOTATION = 'autoscaling.knative.dev/initialScale'
MIN_SCALE_ANNOTATION = 'autoscaling.knative.dev/minScale'
MAX_SCALE_ANNOTATION = 'autoscaling.knative.dev/maxScale'
NVIDIA_GPU_RESOURCE = 'nvidia.com/gpu'
SERVICE_LABEL = 'serving.knative.dev/service'
USER_IMAGE_ANNOTATION = kubernetesobject.CLIENT_GROUP + '/user-image'

USER_IMAGE_PATTERN = re.compile('([^@]+)@sha256:([0-9A-Fa-f]+)')


class Revision(kubernetesobject.KubernetesObject):
  """Wraps the revision field of Knative service."""

  def Kind(self):
    return 'Revision'

  @property
  def labels(self):
    return self.metadata.labels

  @property
  def resource_limits(self):
    return self.container.resources.get('limits', {})

  @property
  def creation_timestamp(self):
    return self.metadata.creationTimestamp

  @property
  def author(self):
    return self.annotations.get(AUTHOR_ANNOTATION)

  @property
  def service_name(self):
    return self.labels[SERVICE_LABEL]

  @property
  def spec(self):
    return Spec(self._props['spec'])

  @property
  def container(self):
    return Container(self.spec.container)

  @property
  def image(self):
    return self.container.image

  @property
  def env_vars(self):
    return self.container.env

  @property
  def concurrency(self):
    return self.spec.concurrency

  @property
  def timeout(self):
    return self.spec.timeout

  # copied from run/revision
  def UserImage(self, service_user_image=None):
    """Human-readable "what's deployed".

    Sometimes references a client.knative.dev/user-image annotation on the
    revision or service to determine what the user intended to deploy. In that
    case, we can display that, and show the user the hash prefix as a note that
    it's at that specific hash.
    TODO (b/168630076): Investigate if this code can be shared with the Run
    binary.

    Arguments:
      service_user_image: Optional[str], the contents of the user image
        annotation on the service.

    Returns:
      A string representing the user deployment intent.
    """
    if not self.image:
      return None
    if '@' not in self.image:
      return self.image
    user_image = (
        self.annotations.get(USER_IMAGE_ANNOTATION) or service_user_image)
    if not user_image:
      return self.image
    # The image should  be in the format base@sha256:hashhashhash
    match = USER_IMAGE_PATTERN.match(self.image)
    if not match:
      return self.image
    (base, h) = match.group(1, 2)
    if not user_image.startswith(base):
      # The user-image is out of date.
      return self.image
    if len(h) > 8:
      h = h[:8] + '...'
    return user_image + ' at ' + h

  @property
  def volumes(self):
    return Volumes(self.spec.volumes)

  @property
  def volume_mounts(self):
    if self.container:
      return VolumeMounts(self.volumes, self.container.volume_mounts)
    else:
      return []

  def MountedVolumeJoin(self, subgroup=None):
    vols = self.volumes
    mounts = self.volume_mounts
    if subgroup:
      vols = getattr(vols, subgroup)
      mounts = getattr(mounts, subgroup)
    return {path: vols.get(vol) for path, vol in mounts.items()}

  @property
  def active(self):
    active_cond = [x for x in self.status.conditions if x.type == 'Active']
    if active_cond:
      return active_cond[0].status
    return None

  def ReadySymbolAndColor(self):
    if not self.ready:
      return '!', 'yellow'
    return super(Revision, self).ReadySymbolAndColor()


class Spec(structuredout.MapObject):
  """Wraps the spec field of the Template resource."""

  @property
  def container(self):
    return self._props['containers'][0]

  @property
  def concurrency(self):
    return self._props.get('containerConcurrency')

  @property
  def timeout(self):
    return self._props.get('timeoutSeconds')

  @property
  def volumes(self):
    if 'volumes' in self._props:
      return self._props['volumes']
    else:
      return []

  @property
  def serviceAccountName(self):
    return self._props.get('serviceAccountName')


class Container(structuredout.MapObject):
  """Wraps a container resource."""

  @property
  def command(self):
    return self._props.get('command', [])

  @property
  def args(self):
    return self._props.get('args', [])

  @property
  def ports(self):
    return [ContainerPort(x) for x in self._props.get('ports', [])]

  @property
  def volume_mounts(self):
    if 'volumeMounts' in self._props:
      return self._props['volumeMounts']
    else:
      return []

  @property
  def image(self):
    return self._props.get('image')

  @property
  def resources(self):
    return self._props.get('resources')

  @property
  def env(self):
    return EnvVars(self._props.get('env', dict()))


class EnvVars():
  """Represents the list of env vars/secrets/config maps.

  Provides properties to access the various type of env vars.
  """

  def __init__(self, env_var_list):
    if env_var_list:
      self._env_var_list = env_var_list
    else:
      self._env_var_list = dict()

  @property
  def literals(self):
    return {
        env['name']: env.get('value')
        for env in self._env_var_list
        if env.get('valueFrom') is None
    }

  @property
  def secrets(self):
    return {
        env['name']: EnvValueFrom(env.get('valueFrom'))
        for env in self._env_var_list
        if env.get('valueFrom') and env.get('valueFrom').get('secretKeyRef')
    }

  @property
  def config_maps(self):
    return {
        env['name']: EnvValueFrom(env.get('valueFrom'))
        for env in self._env_var_list
        if env.get('valueFrom') and env.get('valueFrom').get('configMapKeyRef')
    }


class EnvValueFrom(structuredout.MapObject):
  """Represents the ValueFrom field of an EnvVar."""

  @property
  def secretKeyRef(self):
    if self._props.get('secretKeyRef'):
      return SecretKey(self._props.get('secretKeyRef'))
    else:
      return None

  @property
  def configMapKeyRef(self):
    if self._props.get('configMapKeyRef'):
      return ConfigMapKey(self._props.get('configMapKeyRef'))
    else:
      return None


class Key(structuredout.MapObject):

  @property
  def key(self):
    return self._props.get('key')

  @property
  def name(self):
    return self._props.get('name')


class ConfigMapKey(Key):
  pass


class SecretKey(Key):
  pass


class ContainerPort(structuredout.MapObject):

  @property
  def containerPort(self):
    return self._props.get('containerPort')


class Volumes():
  """Represents the volumes in a revision.spec."""

  def __init__(self, volumes):
    self._volumes = volumes

  @property
  def secrets(self):
    return {
        vol['name']: SecretVolumeItem(vol.get('secret'))
        for vol in self._volumes
        if vol.get('secret')
    }

  @property
  def config_maps(self):
    return {
        vol['name']: ConfigMapVolumeItem(vol.get('configMap'))
        for vol in self._volumes
        if vol.get('configMap')
    }


class VolumeMounts():
  """Represents the volume mounts in a revision.spec.container."""

  def __init__(self, volumes, volume_mounts):
    self._volumes = volumes
    self._volume_mounts = volume_mounts

  @property
  def secrets(self):
    return {
        mount['mountPath']: mount['name']
        for mount in self._volume_mounts
        if mount['name'] in self._volumes.secrets
    }

  @property
  def config_maps(self):
    return {
        mount['mountPath']: mount['name']
        for mount in self._volume_mounts
        if mount['name'] in self._volumes.config_maps
    }


class VolumeItem(structuredout.MapObject):

  @property
  def items(self):
    return [KeyToPath(x) for x in self._props.get('items', [])]


class SecretVolumeItem(VolumeItem):

  @property
  def secretName(self):
    return self._props.get('secretName')


class ConfigMapVolumeItem(VolumeItem):

  @property
  def name(self):
    return self._props.get('name')


class KeyToPath(structuredout.MapObject):

  @property
  def key(self):
    return self._props.get('key')

  @property
  def path(self):
    return self._props.get('path')



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""'vmware vcenter credentials reset' command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.vmware.privateclouds import PrivateCloudsClient
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.vmware import flags
from googlecloudsdk.core import log

DETAILED_HELP = {
    'DESCRIPTION':
        """
          Reset VMware vCenter sign-in credentials associated with a VMware Engine private cloud.
        """,
    'EXAMPLES':
        """
          To reset sign-in credentials for vCenter in private cloud ``my-private-cloud'', run:


            $ {command} --private-cloud=my-private-cloud --location=us-west2-a --project=my-project

          Or:

            $ {command} --private-cloud=my-private-cloud

          In the second example, the project and location are taken from gcloud properties core/project and compute/zone.

    """,
}


@base.Hidden
@base.ReleaseTracks(base.ReleaseTrack.GA)
class Reset(base.UpdateCommand):
  """Reset VMware vCenter sign-in credentials associated with a Google Cloud VMware Engine private cloud."""

  detailed_help = DETAILED_HELP

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    flags.AddPrivatecloudArgToParser(parser)
    base.ASYNC_FLAG.AddToParser(parser)
    base.ASYNC_FLAG.SetDefault(parser, True)

  def Run(self, args):
    resource = args.CONCEPTS.private_cloud.Parse()
    client = PrivateCloudsClient()
    is_async = args.async_
    operation = client.ResetVcenterCredentials(resource)
    if is_async:
      log.UpdatedResource(
          operation.name, kind='vcenter credentials', is_async=True)
      return operation

    resource = client.WaitForOperation(
        operation_ref=client.GetOperationRef(operation),
        message='waiting for vcenter credentials [{}] to be reset'.format(
            resource.RelativeName()))
    log.UpdatedResource(resource, kind='vcenter credentials')
    return resource



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Helper methods for configuring deployment manager command flags."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.deployment_manager import dm_api_util
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.util.apis import arg_utils


RESOURCES_AND_OUTPUTS_FORMAT = """
    table(
      resources:format='table(
        name,
        type:wrap,
        update.state.yesno(no="COMPLETED"),
        update.error.errors.group(code),
        update.intent)',
      outputs:format='table(
        name:label=OUTPUTS,
        finalValue:label=VALUE)'
    )
"""

OPERATION_FORMAT = """
    table(
      name,
      operationType:label=TYPE,
      status,
      targetLink.basename():label=TARGET,
      error.errors.group(code),
      warnings.group(code)
    )
"""

DEPLOYMENT_FORMAT = """
    default(
      name, id, description, fingerprint,insertTime, manifest.basename(),
      labels, operation.operationType, operation.progress,
      operation.status, operation.user, operation.endTime, operation.startTime,
      operation.error, operation.warnings, update)
"""

_DELETE_FLAG_KWARGS = {
    'help_str': ('Delete policy for resources that will change as part of '
                 'an update or delete. `delete` deletes the resource while '
                 '`abandon` just removes the resource reference from the '
                 'deployment.'),
    'default': 'delete',
    'name': '--delete-policy'
}


def GetDeleteFlagEnumMap(policy_enum):
  return arg_utils.ChoiceEnumMapper(
      _DELETE_FLAG_KWARGS['name'],
      policy_enum,
      help_str=_DELETE_FLAG_KWARGS['help_str'],
      default=_DELETE_FLAG_KWARGS['default'])


def AddDeploymentNameFlag(parser):
  """Add properties flag."""
  parser.add_argument('deployment_name', help='Deployment name.')


def AddConfigFlags(parser):
  """Add flags for different types of configs."""
  parser.add_argument(
      '--config',
      help='Filename of a top-level yaml config that specifies '
      'resources to deploy. For a guide to creating a configuration, refer '
      'to https://cloud.google.com/deployment-manager/docs/configuration/'
      'create-basic-configuration')

  parser.add_argument(
      '--template',
      help='Filename of a top-level jinja or python config template.')

  parser.add_argument(
      '--composite-type',
      help='Name of a composite type to deploy. For an example of creating '
      'and deploying a composite type, see: https://cloud.google.com/'
      'deployment-manager/docs/configuration/templates/create-composite-types'
      '#examplecompositetype')


def AddPropertiesFlag(parser):
  """Add properties flag."""

  parser.add_argument(
      '--properties',
      help='A comma separated, key:value, map '
      'to be used when deploying a template file or composite type directly.',
      type=arg_parsers.ArgDict(operators=dm_api_util.NewParserDict()),
      dest='properties')


def AddAsyncFlag(parser):
  """Add the async argument."""
  base.ASYNC_FLAG.AddToParser(parser)


def AddFingerprintFlag(parser):
  """Add the fingerprint argument."""
  parser.add_argument(
      '--fingerprint',
      help=('The fingerprint to use in requests to modify a deployment. If not '
            'specified, a get deployment request will be made to fetch the '
            'latest fingerprint. A fingerprint is a randomly generated value '
            'that is part of the update, stop, and cancel-preview request to '
            'perform optimistic locking. It is initially generated by '
            'Deployment Manager and changes after every request to modify '
            'data. The latest fingerprint is printed when deployment data is '
            'modified.'),
      dest='fingerprint')



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Helpers to validate config set values."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import exceptions as apitools_exceptions

from googlecloudsdk.api_lib.cloudresourcemanager import projects_api
from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import lister
from googlecloudsdk.api_lib.util import exceptions as api_lib_util_exceptions
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.projects import util as command_lib_util
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core.credentials import store as c_store


def WarnIfSettingNonExistentRegionZone(value, zonal=True):
  """Warn if setting 'compute/region' or 'compute/zone' to wrong value."""
  zonal_msg = ('{} is not a valid zone. Run `gcloud compute zones list` to '
               'get all zones.'.format(value))
  regional_msg = ('{} is not a valid region. Run `gcloud compute regions list`'
                  'to get all regions.'.format(value))
  holder = base_classes.ComputeApiHolder(base.ReleaseTrack.GA)
  client = holder.client
  # pylint: disable=protected-access
  request_data = lister._Frontend(None, None, lister.GlobalScope([
      holder.resources.Parse(
          properties.VALUES.core.project.GetOrFail(),
          collection='compute.projects')
  ]))

  list_implementation = lister.GlobalLister(
      client, client.apitools_client.zones if zonal
      else client.apitools_client.regions)
  try:
    response = lister.Invoke(request_data, list_implementation)
    zones = [i['name'] for i in list(response)]
    if value not in zones:
      log.warning(zonal_msg if zonal else regional_msg)
      return True
  except (lister.ListException,
          apitools_exceptions.HttpError,
          c_store.NoCredentialsForAccountException,
          api_lib_util_exceptions.HttpException):
    log.warning('Property validation for compute/{} was skipped.'.format(
        'zone' if zonal else 'region'))
  return False


def WarnIfSettingProjectWithNoAccess(scope, project):
  """Warn if setting 'core/project' config to inaccessible project."""

  # Only display a warning if the following conditions are true:
  #
  # * The current scope is USER (not occurring in the context of installation).
  # * The 'core/account' value is set (a user has authed).
  #
  # If the above conditions are met, check that the project being set exists
  # and is accessible to the current user, otherwise show a warning.
  if (scope == properties.Scope.USER and
      properties.VALUES.core.account.Get()):
    project_ref = command_lib_util.ParseProject(project)
    try:
      with base.WithLegacyQuota():
        projects_api.Get(project_ref, disable_api_enablement_check=True)
    except (apitools_exceptions.HttpError,
            c_store.NoCredentialsForAccountException,
            api_lib_util_exceptions.HttpException):
      log.warning(
          'You do not appear to have access to project [{}] or'
          ' it does not exist.'.format(project))
      return True
  return False


def WarnIfActivateUseClientCertificate(value):
  """Warns if setting context_aware/use_client_certificate to truthy."""
  if value.lower() in ['1', 'true', 'on', 'yes', 'y']:
    mtls_not_supported_msg = (
        'Some services may not support client certificate authorization in '
        'this version of gcloud. When a command sends requests to such '
        'services, the requests will be executed without using a client '
        'certificate.\n\n'
        'Please run $ gcloud topic client-certificate for more information.')
    log.warning(mtls_not_supported_msg)



"""Generated message classes for orgpolicy version v2alpha1.

The Org Policy API allows users to configure governance rules on their GCP
resources across the Cloud Resource Hierarchy.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding


package = 'orgpolicy'


class GoogleCloudOrgpolicyV2alpha1Constraint(_messages.Message):
  r"""A `Constraint` describes a way in which a resource's configuration can
  be restricted. For example, it controls which cloud services can be
  activated across an organization, or whether a Compute Engine instance can
  have serial port connections established. `Constraints` can be configured by
  the organization's policy adminstrator to fit the needs of the organzation
  by setting Policies for `Constraints` at different locations in the
  organization's resource hierarchy. Policies are inherited down the resource
  hierarchy from higher levels, but can also be overridden. For details about
  the inheritance rules please read about Policies. `Constraints` have a
  default behavior determined by the `constraint_default` field, which is the
  enforcement behavior that is used in the absence of a `Policy` being defined
  or inherited for the resource in question.

  Enums:
    ConstraintDefaultValueValuesEnum: The evaluation behavior of this
      constraint in the absense of 'Policy'.

  Fields:
    booleanConstraint: Defines this constraint as being a BooleanConstraint.
    constraintDefault: The evaluation behavior of this constraint in the
      absense of 'Policy'.
    description: Detailed description of what this `Constraint` controls as
      well as how and where it is enforced. Mutable.
    displayName: The human readable name. Mutable.
    listConstraint: Defines this constraint as being a ListConstraint.
    name: The resource name of the Constraint. Must be in one of the following
      forms: * `projects/{project_number}/constraints/{constraint_name}` *
      `folders/{folder_id}/constraints/{constraint_name}` *
      `organizations/{organization_id}/constraints/{constraint_name}` For
      example, "/projects/123/constraints/compute.disableSerialPortAccess".
      Immutable.
  """

  class ConstraintDefaultValueValuesEnum(_messages.Enum):
    r"""The evaluation behavior of this constraint in the absense of 'Policy'.

    Values:
      CONSTRAINT_DEFAULT_UNSPECIFIED: This is only used for distinguishing
        unset values and should never be used.
      ALLOW: Indicate that all values are allowed for list constraints.
        Indicate that enforcement is off for boolean constraints.
      DENY: Indicate that all values are denied for list constraints. Indicate
        that enforcement is on for boolean constraints.
    """
    CONSTRAINT_DEFAULT_UNSPECIFIED = 0
    ALLOW = 1
    DENY = 2

  booleanConstraint = _messages.MessageField('GoogleCloudOrgpolicyV2alpha1ConstraintBooleanConstraint', 1)
  constraintDefault = _messages.EnumField('ConstraintDefaultValueValuesEnum', 2)
  description = _messages.StringField(3)
  displayName = _messages.StringField(4)
  listConstraint = _messages.MessageField('GoogleCloudOrgpolicyV2alpha1ConstraintListConstraint', 5)
  name = _messages.StringField(6)


class GoogleCloudOrgpolicyV2alpha1ConstraintBooleanConstraint(_messages.Message):
  r"""A `Constraint` that is either enforced or not. For example a constraint
  `constraints/compute.disableSerialPortAccess`. If it is enforced on a VM
  instance, serial port connections will not be opened to that instance.
  """



class GoogleCloudOrgpolicyV2alpha1ConstraintListConstraint(_messages.Message):
  r"""A `Constraint` that allows or disallows a list of string values, which
  are configured by an Organization's policy administrator with a `Policy`.

  Fields:
    supportsExpr: Indicates whether CEL expression values can be used in
      `Policy.allowed_values` and `Policy.denied_values`. For example,
      "expr:value.matches('a.b')" would match an input value that matches the
      regular expression 'a.b'.
    supportsIn: Indicates whether values grouped into categories can be used
      in `Policy.allowed_values` and `Policy.denied_values`. For example,
      `"in:Python"` would match any value in the 'Python' group.
    supportsUnder: Indicates whether subtrees of Cloud Resource Manager
      resource hierarchy can be used in `Policy.allowed_values` and
      `Policy.denied_values`. For example, `"under:folders/123"` would match
      any resource under the 'folders/123' folder.
  """

  supportsExpr = _messages.BooleanField(1)
  supportsIn = _messages.BooleanField(2)
  supportsUnder = _messages.BooleanField(3)


class GoogleCloudOrgpolicyV2alpha1ListConstraintsResponse(_messages.Message):
  r"""The response returned from the ListConstraints method.

  Fields:
    constraints: The collection of constraints that are available on the
      targeted resource.
    nextPageToken: Page token used to retrieve the next page. This is
      currently not used.
  """

  constraints = _messages.MessageField('GoogleCloudOrgpolicyV2alpha1Constraint', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class GoogleCloudOrgpolicyV2alpha1ListPoliciesResponse(_messages.Message):
  r"""The response returned from the ListPolicies method. It will be empty if
  no `Policies` are set on the resource.

  Fields:
    nextPageToken: Page token used to retrieve the next page. This is
      currently not used, but the server may at any point start supplying a
      valid token.
    policies: All `Policies` that exist on the resource. It will be empty if
      no `Policies` are set.
  """

  nextPageToken = _messages.StringField(1)
  policies = _messages.MessageField('GoogleCloudOrgpolicyV2alpha1Policy', 2, repeated=True)


class GoogleCloudOrgpolicyV2alpha1Policy(_messages.Message):
  r"""Defines a Cloud Organization `Policy` which is used to specify
  `Constraints` for configurations of Cloud Platform resources.

  Fields:
    dryRunSpec: dry-run policy. go/op-dryrun-dd
    name: Immutable. The resource name of the Policy. Must be one of the
      following forms, where constraint_name is the name of the constraint
      which this Policy configures: *
      `projects/{project_number}/policies/{constraint_name}` *
      `folders/{folder_id}/policies/{constraint_name}` *
      `organizations/{organization_id}/policies/{constraint_name}` For
      example, "projects/123/policies/compute.disableSerialPortAccess". Note:
      `projects/{project_id}/policies/{constraint_name}` is also an acceptable
      name for API requests, but responses will return the name using the
      equivalent project number.
    spec: A GoogleCloudOrgpolicyV2alpha1PolicySpec attribute.
  """

  dryRunSpec = _messages.MessageField('GoogleCloudOrgpolicyV2alpha1PolicySpec', 1)
  name = _messages.StringField(2)
  spec = _messages.MessageField('GoogleCloudOrgpolicyV2alpha1PolicySpec', 3)


class GoogleCloudOrgpolicyV2alpha1PolicySpec(_messages.Message):
  r"""Defines a Cloud Organization `PolicySpec` which is used to specify
  `Constraints` for configurations of Cloud Platform resources.

  Fields:
    etag: An opaque tag indicating the current version of the `Policy`, used
      for concurrency control. This field is ignored if used in a
      `CreatePolicy` request. When the `Policy` is returned from either a
      `GetPolicy` or a `ListPolicies` request, this `etag` indicates the
      version of the current `Policy` to use when executing a read-modify-
      write loop. When the `Policy` is returned from a `GetEffectivePolicy`
      request, the `etag` will be unset. When the `Policy` is used in a
      `UpdatePolicy` method, use the `etag` value that was returned from a
      `GetPolicy` request as part of a read-modify-write loop for concurrency
      control. If `UpdatePolicyRequest`'s `force_unconditional_write` field is
      set to true, this field must not be set. Otherwise, the `etag` is
      required for `UpdatePolicy`.
    inheritFromParent: This field can be set only for Policies which configure
      list constraints.
    reset: Ignores policies set above this resource and restores the
      `constraint_default` enforcement behavior of the specific `Constraint`
      at this resource. This field can be set in policies for either list or
      boolean constraints. If set, `rules` must be empty and
      `inherit_from_parent` must be set to false.
    rules: Up to 10 PolicyRules are allowed. In Policies for boolean
      constraints, the following requirements apply: - There must be one and
      only one PolicyRule where condition is unset. - BooleanPolicyRules with
      conditions must set `enforced` to the opposite of the PolicyRule without
      a condition. - During policy evaluation, PolicyRules with conditions
      that are true for a target resource take precedence.
    updateTime: Output only. The time stamp this was previously updated. This
      represents the last time a call to `CreatePolicy` or `UpdatePolicy` was
      made for that `Policy`.
  """

  etag = _messages.StringField(1)
  inheritFromParent = _messages.BooleanField(2)
  reset = _messages.BooleanField(3)
  rules = _messages.MessageField('GoogleCloudOrgpolicyV2alpha1PolicySpecPolicyRule', 4, repeated=True)
  updateTime = _messages.StringField(5)


class GoogleCloudOrgpolicyV2alpha1PolicySpecPolicyRule(_messages.Message):
  r"""A rule used to express this policy.

  Fields:
    allowAll: Setting this to true means that all values are allowed. This
      field can be set only in Policies for list constraints.
    condition: Optional. A condition which determines whether this rule is
      used in the evaluation of the policy. When set, the `expression` field
      in the `Expr' must include from 1 to 10 subexpressions, joined by the
      "||" or "&&" operators. Each subexpression must be of the form
      "resource.matchLabels(key_name, value_name)", where key_name and
      value_name are the resource names for Label Keys and Values. These names
      are available from the Label Manager Service. An example expression is:
      "resource.matchLabels('labelKeys/123, 'labelValues/456')".
    denyAll: Setting this to true means that all values are denied. This field
      can be set only in Policies for list constraints.
    enforce: If `true`, then the `Policy` is enforced. If `false`, then any
      configuration is acceptable. This field can be set only in Policies for
      boolean constraints.
    values: List of values to be used for this PolicyRule. This field can be
      set only in Policies for list constraints.
  """

  allowAll = _messages.BooleanField(1)
  condition = _messages.MessageField('GoogleTypeExpr', 2)
  denyAll = _messages.BooleanField(3)
  enforce = _messages.BooleanField(4)
  values = _messages.MessageField('GoogleCloudOrgpolicyV2alpha1PolicySpecPolicyRuleStringValues', 5)


class GoogleCloudOrgpolicyV2alpha1PolicySpecPolicyRuleStringValues(_messages.Message):
  r"""A message that holds specific allowed and denied values. This message
  can define specific values and subtrees of Cloud Resource Manager resource
  hierarchy (`Organizations`, `Folders`, `Projects`) that are allowed or
  denied. This is achieved by using the `under:` and optional `is:` prefixes.
  The `under:` prefix is used to denote resource subtree values. The `is:`
  prefix is used to denote specific values, and is required only if the value
  contains a ":". Values prefixed with "is:" are treated the same as values
  with no prefix. Ancestry subtrees must be in one of the following formats: -
  "projects/", e.g. "projects/tokyo-rain-123" - "folders/", e.g.
  "folders/1234" - "organizations/", e.g. "organizations/1234" The
  `supports_under` field of the associated `Constraint` defines whether
  ancestry prefixes can be used.

  Fields:
    allowedValues: List of values allowed at this resource.
    deniedValues: List of values denied at this resource.
  """

  allowedValues = _messages.StringField(1, repeated=True)
  deniedValues = _messages.StringField(2, repeated=True)


class GoogleProtobufEmpty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class GoogleTypeExpr(_messages.Message):
  r"""Represents a textual expression in the Common Expression Language (CEL)
  syntax. CEL is a C-like expression language. The syntax and semantics of CEL
  are documented at https://github.com/google/cel-spec. Example (Comparison):
  title: "Summary size limit" description: "Determines if a summary is less
  than 100 chars" expression: "document.summary.size() < 100" Example
  (Equality): title: "Requestor is owner" description: "Determines if
  requestor is the document owner" expression: "document.owner ==
  request.auth.claims.email" Example (Logic): title: "Public documents"
  description: "Determine whether the document should be publicly visible"
  expression: "document.type != 'private' && document.type != 'internal'"
  Example (Data Manipulation): title: "Notification string" description:
  "Create a notification string with a timestamp." expression: "'New message
  received at ' + string(document.create_time)" The exact variables and
  functions that may be referenced within an expression are determined by the
  service that evaluates it. See the service documentation for additional
  information.

  Fields:
    description: Optional. Description of the expression. This is a longer
      text which describes the expression, e.g. when hovered over it in a UI.
    expression: Textual representation of an expression in Common Expression
      Language syntax.
    location: Optional. String indicating the location of the expression for
      error reporting, e.g. a file name and a position in the file.
    title: Optional. Title for the expression, i.e. a short string describing
      its purpose. This can be used e.g. in UIs which allow to enter the
      expression.
  """

  description = _messages.StringField(1)
  expression = _messages.StringField(2)
  location = _messages.StringField(3)
  title = _messages.StringField(4)


class OrgpolicyConstraintsListRequest(_messages.Message):
  r"""A OrgpolicyConstraintsListRequest object.

  Fields:
    pageSize: Size of the pages to be returned. This is currently unsupported
      and will be ignored. The server may at any point start using this field
      to limit page size.
    pageToken: Page token used to retrieve the next page. This is currently
      unsupported and will be ignored. The server may at any point start using
      this field.
    parent: The Cloud resource that parents the constraint. Must be in one of
      the following forms: * `projects/{project_number}` *
      `projects/{project_id}` * `folders/{folder_id}` *
      `organizations/{organization_id}`
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class OrgpolicyPoliciesCreateRequest(_messages.Message):
  r"""A OrgpolicyPoliciesCreateRequest object.

  Fields:
    constraint: The name of the `Constraint` the `Policy` is configuring, for
      example, `constraints/compute.disableSerialPortAccess`.
    googleCloudOrgpolicyV2alpha1Policy: A GoogleCloudOrgpolicyV2alpha1Policy
      resource to be passed as the request body.
    parent: The Cloud resource that will parent the new Policy. Must be in one
      of the following forms: * `projects/{project_number}` *
      `projects/{project_id}` * `folders/{folder_id}` *
      `organizations/{organization_id}`
  """

  constraint = _messages.StringField(1)
  googleCloudOrgpolicyV2alpha1Policy = _messages.MessageField('GoogleCloudOrgpolicyV2alpha1Policy', 2)
  parent = _messages.StringField(3, required=True)


class OrgpolicyPoliciesDeleteRequest(_messages.Message):
  r"""A OrgpolicyPoliciesDeleteRequest object.

  Fields:
    name: Name of the policy to delete. See `Policy` for naming rules.
  """

  name = _messages.StringField(1, required=True)


class OrgpolicyPoliciesGetEffectivePolicyRequest(_messages.Message):
  r"""A OrgpolicyPoliciesGetEffectivePolicyRequest object.

  Fields:
    name: The effective policy to compute. See `Policy` for naming rules.
  """

  name = _messages.StringField(1, required=True)


class OrgpolicyPoliciesGetRequest(_messages.Message):
  r"""A OrgpolicyPoliciesGetRequest object.

  Fields:
    name: Resource name of the policy. See `Policy` for naming requirements.
  """

  name = _messages.StringField(1, required=True)


class OrgpolicyPoliciesListRequest(_messages.Message):
  r"""A OrgpolicyPoliciesListRequest object.

  Fields:
    pageSize: Size of the pages to be returned. This is currently unsupported
      and will be ignored. The server may at any point start using this field
      to limit page size.
    pageToken: Page token used to retrieve the next page. This is currently
      unsupported and will be ignored. The server may at any point start using
      this field.
    parent: The target Cloud resource that parents the set of constraints and
      policies that will be returned from this call. Must be in one of the
      following forms: * `projects/{project_number}` * `projects/{project_id}`
      * `folders/{folder_id}` * `organizations/{organization_id}`
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class OrgpolicyPoliciesPatchRequest(_messages.Message):
  r"""A OrgpolicyPoliciesPatchRequest object.

  Fields:
    forceUnconditionalWrite: Indicates that this request should overwrite the
      current policy and ignore the `etag` used for optimistic currency
      control. When set to true, `etag` in `Policy` must be unset. Otherwise,
      `etag` is required.
    googleCloudOrgpolicyV2alpha1Policy: A GoogleCloudOrgpolicyV2alpha1Policy
      resource to be passed as the request body.
    name: Immutable. The resource name of the Policy. Must be one of the
      following forms, where constraint_name is the name of the constraint
      which this Policy configures: *
      `projects/{project_number}/policies/{constraint_name}` *
      `folders/{folder_id}/policies/{constraint_name}` *
      `organizations/{organization_id}/policies/{constraint_name}` For
      example, "projects/123/policies/compute.disableSerialPortAccess". Note:
      `projects/{project_id}/policies/{constraint_name}` is also an acceptable
      name for API requests, but responses will return the name using the
      equivalent project number.
  """

  forceUnconditionalWrite = _messages.BooleanField(1)
  googleCloudOrgpolicyV2alpha1Policy = _messages.MessageField('GoogleCloudOrgpolicyV2alpha1Policy', 2)
  name = _messages.StringField(3, required=True)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2019 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Common stateful utilities for the gcloud Life Sciences tool."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.util import apis

OPERATION_TIMEOUT = 60 * 20 * 1000


class LifeSciencesClient(object):
  """Client for calling Cloud Life Sciences APIs."""

  def __init__(self):
    super(LifeSciencesClient, self).__init__()
    self._api_version = 'v2beta'
    self._client = None
    self._resources = None

  @property
  def client(self):
    if self._client is None:
      self._client = apis.GetClientInstance('lifesciences', self._api_version)
    return self._client

  @property
  def messages(self):
    return self.client.MESSAGES_MODULE



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A persistent cache implementation using files.

See the persistent_cache module for a detailed description.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import errno
import fnmatch
import json
import os

from googlecloudsdk.core.cache import exceptions
from googlecloudsdk.core.cache import metadata_table
from googlecloudsdk.core.cache import persistent_cache_base
from googlecloudsdk.core.util import files

import six
from six.moves import range  # pylint: disable=redefined-builtin


class _Table(persistent_cache_base.Table):
  """A persistent cache table.

  Attributes:
    name: The table name.
    deleted: Table was deleted if True.
    restricted: Table is restricted if True.
    modified: Table modify timestamp.
    timeout: Tables older than timeout are invalid.
    _cache: The parent cache object.
    _rows: The list of rows in the table.
  """

  def __init__(self, cache, name, columns=1, keys=1, timeout=0, modified=0,
               restricted=False):
    self._rows = None
    super(_Table, self).__init__(cache, name, columns=columns, keys=keys,
                                 timeout=timeout, modified=modified,
                                 restricted=restricted)
    if restricted:
      self._cache._restricted.add(name)  # pylint: disable=protected-access
    self.deleted = False
    try:
      contents = files.ReadFileContents(
          os.path.join(self._cache.name, self.EncodeName(name)))
    except files.MissingFileError:
      contents = None
      self.changed = True
    except files.Error:
      raise

    if contents:
      self._rows = [tuple(r) for r in json.loads(contents)]
    else:
      self._rows = []
    # pylint: disable=protected-access
    if self._cache._metadata:
      self._cache._tables[name] = self

  def Delete(self):
    """Deletes the table."""
    self.Invalidate()
    self.DeleteRows()
    # pylint: disable=protected-access
    self._cache._metadata.DeleteRows([(self.name,)])
    self.deleted = True

  def _Commit(self):
    """Commits changed/deleted table data to the table file."""
    if self.changed:
      self.changed = False
      path = os.path.join(self._cache.name, self.EncodeName(self.name))
      # pylint: disable=protected-access
      if self.deleted:
        self.deleted = False
        self._cache._metadata.DeleteRows([(self.name,)])
        del self._cache._tables[self.name]
        try:
          os.remove(path)
        except OSError as e:
          # The deleted table might have never been committed.
          if e.errno != errno.ENOENT:
            raise
      else:
        self._cache._metadata.AddRows(
            [metadata_table.Metadata.Row(
                name=self.name,
                columns=self.columns,
                keys=self.keys,
                timeout=self.timeout,
                modified=self.modified,
                restricted=self.restricted,
                version=self._cache.version)])
        files.WriteFileContents(path, json.dumps(self._rows))

  def _RowEqual(self, a, b):
    """Returns True if rows a and b have the same key."""
    return a[:self.keys] == b[:self.keys]

  def _RowMatch(self, row_template, row):
    """Returns True if row_template matches row."""
    if row_template:
      for i in range(len(row_template)):
        if row_template[i] is not None:
          if (isinstance(row_template[i], six.string_types) and
              isinstance(row[i], six.string_types)):
            if not fnmatch.fnmatch(row[i], row_template[i]):
              return False
          elif row_template[i] != row[i]:
            return False
    return True

  def _AnyRowMatch(self, row_templates, row):
    """Returns True if any template in row_templates matches row."""
    for row_template in row_templates:
      if self._RowMatch(row_template, row):
        return True
    return False

  def AddRows(self, rows):
    """Adds each row in rows to the table."""
    self._CheckRows(rows)
    self.changed = True
    rows = sorted(self._rows + list(rows), key=lambda x: x[:self.keys])
    self._rows = []
    i = 0
    while i < len(rows):
      # Skip over dup keys, keep the last, which is a new entry thanks to the
      # stable sort above.
      while i < len(rows) - 1 and self._RowEqual(rows[i], rows[i + 1]):
        i += 1
      self._rows.append(rows[i])
      i += 1

  def DeleteRows(self, row_templates=None):
    """Deletes each row in the table matching any of the row_templates."""
    self.changed = True
    if row_templates:
      self._CheckRowTemplates(row_templates)
      keep = []
      for row in self._rows:
        if not self._AnyRowMatch(row_templates, row):
          keep.append(row)
      self._rows = keep
    else:
      self._rows = []

  def Select(self, row_template=None, ignore_expiration=False):
    """Returns the list of rows that match row_template, None for all."""
    if row_template is not None:
      self._CheckRowTemplates([row_template])
    if not ignore_expiration and not self.restricted and not self.modified:
      raise exceptions.CacheTableExpired(
          '[{}] cache table [{}] has expired.'.format(
              self._cache.name, self.name))
    matched = []
    for row in self._rows:
      if row and self._RowMatch(row_template, row):
        matched.append(row)
    return matched


class Cache(metadata_table.CacheUsingMetadataTable):
  """A persistent cache object.

  Attributes:
    name: The db path name. Created/removed by this object. May be a file or
      directory. In this implementation its a file.
    timeout: The default table timeout.
    version: A caller defined version string that must match the version string
      stored when the persistent object was created.
    _lock: The cache lock object. None if no files have been committed yet.
    _lock_path: The cache lock meta file.
    _metadata: The metadata restricted _Table.
    _persistent: True if the persistent object has been committed at least once.
    _restricted: The set of restricted table names.
    _start: The cache instance start time.
    _tables: The map of open table objects.
  """

  def __init__(self, name, create=True, timeout=None, version=None):
    super(Cache, self).__init__(
        _Table, name, create=create, timeout=timeout, version=version)
    lock_name = '__lock__'
    self._restricted = set([lock_name])
    self._tables = {}
    self._metadata = None
    self._start = persistent_cache_base.Now()
    self._lock_path = os.path.join(self.name, lock_name)
    self._lock = None
    self._persistent = False
    if not os.path.exists(self.name):
      if not create:
        raise exceptions.CacheNotFound(
            'Persistent cache [{}] not found.'.format(self.name))
    elif not os.path.exists(self._lock_path):
      raise exceptions.CacheInvalid(
          '[{}] is not a persistent cache.'.format(self.name))
    else:
      # self.name exists and is a directory, and self._lock_path exists.
      self._persistent = True
      self._lock = files.FileLock(self._lock_path, timeout_secs=2)
      self._lock.Lock()
    try:
      self.InitializeMetadata()
    except exceptions.Error:
      # Make sure we clean up any dangling resources.
      self.Close(commit=False)
      raise

  def Delete(self):
    """Permanently deletes the persistent cache."""
    self.Close(commit=False)
    if self._persistent:
      files.RmTree(self.name)
      self._persistent = False

  def Commit(self):
    """Commits all operations up to this point."""
    if not self._lock:
      os.mkdir(self.name, 0o700)
      self._persistent = True
      self._lock = files.FileLock(self._lock_path, timeout_secs=2)
      self._lock.Lock()
    # Update the changed tables.
    for table in list([x for x in self._tables.values() if x.changed]):
      table._Commit()  # pylint: disable=protected-access
    if self._metadata.changed:
      self._metadata._Commit()  # pylint: disable=protected-access

  def Close(self, commit=True):
    """Closes the cache, optionally committing any changes.

    Args:
      commit: Commits any changes before closing if True.
    """
    if commit:
      self.Commit()
    if self._lock:
      self._lock.Unlock()
      self._lock = None
    self._metadata = None
    self._tables = None



# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""API library for access context manager policies."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import list_pager

from googlecloudsdk.api_lib.accesscontextmanager import util
from googlecloudsdk.api_lib.util import waiter

from googlecloudsdk.core import resources


class Client(object):
  """Client for Access Context Manager Access Policies service."""

  def __init__(self, client=None, messages=None, version=None):
    self.client = client or util.GetClient(version=version)
    self.messages = messages or self.client.MESSAGES_MODULE

  def List(self, organization_ref, limit=None):
    req = self.messages.AccesscontextmanagerAccessPoliciesListRequest(
        parent=organization_ref.RelativeName())
    return list_pager.YieldFromList(
        self.client.accessPolicies, req,
        limit=limit,
        batch_size_attribute='pageSize',
        batch_size=None,
        field='accessPolicies')

  def Patch(self, policy_ref, title=None):
    """Patch an access policy.

    Args:
      policy_ref: resources.Resource, reference to the policy to patch
      title: str, title of the policy or None if not updating

    Returns:
      AccessPolicy, the updated access policy
    """
    policy = self.messages.AccessPolicy()
    update_mask = []
    if title is not None:
      update_mask.append('title')
      policy.title = title
    update_mask.sort()  # For ease-of-testing

    m = self.messages
    request_type = m.AccesscontextmanagerAccessPoliciesPatchRequest
    request = request_type(
        accessPolicy=policy,
        name=policy_ref.RelativeName(),
        updateMask=','.join(update_mask),
    )
    operation = self.client.accessPolicies.Patch(request)

    poller = waiter.CloudOperationPoller(self.client.accessPolicies,
                                         self.client.operations)
    poller = util.OperationPoller(
        self.client.accessPolicies, self.client.operations, policy_ref)
    operation_ref = resources.REGISTRY.Parse(
        operation.name, collection='accesscontextmanager.operations')
    return waiter.WaitFor(
        poller, operation_ref,
        'Waiting for PATCH operation [{}]'.format(operation_ref.Name()))



# -*- coding: utf-8 -*- #
# Copyright 2013 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The edit module allows you to edit a text blob without leaving the shell.

When a user needs to edit a blob of text and you don't want to save to
some location, tell them about it, and have the user re-upload the file, this
module can be used to do a quick inline edit.

It will inspect the environment variable EDITOR to see what tool to use
for editing, defaulting to vi. Then, the EDITOR will be opened in the current
terminal; when it exits, the file will be reread and returned with any edits
that the user may have saved while in the EDITOR.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import os
import subprocess
import tempfile

from googlecloudsdk.core.util import encoding
from googlecloudsdk.core.util import files
from googlecloudsdk.core.util import platforms


class Error(Exception):
  """Exceptions for this module."""


class NoSaveException(Error):
  """NoSaveException is thrown when the user did not save the file."""


class EditorException(Error):
  """EditorException is thrown when the editor returns a non-zero exit code."""


def FileModifiedTime(file_name):
  """Enables mocking in the unit test."""
  return os.stat(file_name).st_mtime


def SubprocessCheckCall(*args, **kargs):
  """Enables mocking in the unit test."""
  return subprocess.check_call(*args, **kargs)


def OnlineEdit(text):
  """Edit will edit the provided text.

  Args:
    text: The initial text blob to provide for editing.

  Returns:
    The edited text blob.

  Raises:
    NoSaveException: If the user did not save the temporary file.
    EditorException: If the process running the editor has a
        problem.
  """
  fname = tempfile.NamedTemporaryFile(suffix='.txt').name
  files.WriteFileContents(fname, text)

  # Get the mod time, so we can check if anything was actually done.
  start_mtime = FileModifiedTime(fname)
  if (platforms.OperatingSystem.Current() is
      platforms.OperatingSystem.WINDOWS):
    try:
      SubprocessCheckCall([fname], shell=True)
    except subprocess.CalledProcessError as error:
      raise EditorException('Your editor exited with return code {0}; '
                            'please try again.'.format(error.returncode))
  else:
    try:
      editor = encoding.GetEncodedValue(os.environ, 'EDITOR', 'vi')
      # We use shell=True and manual smashing of the args to permit users to set
      # EDITOR="emacs -nw", or similar things.
      # We used suprocess.check_call instead of subprocess.check_output because
      # subprocess.check_output requires a direct connection to a terminal.
      SubprocessCheckCall('{editor} {file}'.format(
          editor=editor, file=fname), shell=True)
    except subprocess.CalledProcessError as error:
      raise EditorException('Your editor exited with return code {0}; '
                            'please try again. You may set the EDITOR '
                            'environment to use a different text '
                            'editor.'.format(error.returncode))
  end_mtime = FileModifiedTime(fname)
  if start_mtime == end_mtime:
    raise NoSaveException('edit aborted by user')

  return files.ReadFileContents(fname)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Task for streaming downloads.

Typically executed in a task iterator:
googlecloudsdk.command_lib.storage.tasks.task_executor.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import os
import sys
import threading

from googlecloudsdk.api_lib.storage import api_factory
from googlecloudsdk.api_lib.storage import cloud_api
from googlecloudsdk.api_lib.storage import request_config_factory
from googlecloudsdk.command_lib.storage import progress_callbacks
from googlecloudsdk.command_lib.storage.tasks import task
from googlecloudsdk.command_lib.storage.tasks import task_status
from googlecloudsdk.core import log


class StreamingDownloadTask(task.Task):
  """Represents a command operation triggering a streaming download."""

  def __init__(self,
               source_resource,
               download_stream,
               print_created_message=False,
               user_request_args=None,
               show_url=False,
               start_byte=0,
               end_byte=None):
    """Initializes task.

    Args:
      source_resource (ObjectResource): Must contain the full path of object to
        download, including bucket. Directories will not be accepted. Does not
        need to contain metadata.
      download_stream (stream): Reusable stream to write download to.
      print_created_message (bool): Print a message containing the versioned
        URL of the copy result.
      user_request_args (UserRequestArgs|None): Values for RequestConfig.
      show_url (bool): Says whether or not to print the header before each
        object's content
      start_byte (int): The byte index to start streaming from.
      end_byte (int|None): The byte index to stop streaming from.
    """
    super(StreamingDownloadTask, self).__init__()
    self._source_resource = source_resource
    self._download_stream = download_stream
    self._print_created_message = print_created_message
    self._user_request_args = user_request_args
    self._show_url = show_url
    self._start_byte = start_byte
    self._end_byte = end_byte

  def execute(self, task_status_queue=None):
    if self._show_url:
      sys.stderr.write('==> {} <==\n'.format(self._source_resource))
    if task_status_queue:
      progress_callback = progress_callbacks.FilesAndBytesProgressCallback(
          status_queue=task_status_queue,
          offset=0,
          length=self._source_resource.size,
          source_url=self._source_resource.storage_url,
          destination_url=self._download_stream.name,
          operation_name=task_status.OperationName.DOWNLOADING,
          process_id=os.getpid(),
          thread_id=threading.get_ident(),
      )
    else:
      progress_callback = None

    if (self._source_resource.size and
        self._start_byte >= self._source_resource.size):
      if progress_callback:
        progress_callback(self._source_resource.size)
      return

    request_config = request_config_factory.get_request_config(
        self._source_resource.storage_url,
        decryption_key_hash=self._source_resource.decryption_key_hash,
        user_request_args=self._user_request_args,
    )

    provider = self._source_resource.storage_url.scheme
    api_factory.get_api(provider).download_object(
        self._source_resource,
        self._download_stream,
        request_config,
        download_strategy=cloud_api.DownloadStrategy.ONE_SHOT,
        progress_callback=progress_callback,
        start_byte=self._start_byte,
        end_byte=self._end_byte)
    self._download_stream.flush()
    if self._print_created_message:
      log.status.Print('Created: {}'.format(self._download_stream.name))

  def __eq__(self, other):
    if not isinstance(other, self.__class__):
      return NotImplemented
    return (self._source_resource == other._source_resource and
            self._download_stream == other._download_stream and
            self._print_created_message == other._print_created_message and
            self._user_request_args == other._user_request_args and
            self._show_url == other._show_url and
            self._start_byte == other._start_byte and
            self._end_byte == other._end_byte)



"""Generated client library for resourcesettings version v1alpha1."""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.py import base_api
from googlecloudsdk.third_party.apis.resourcesettings.v1alpha1 import resourcesettings_v1alpha1_messages as messages


class ResourcesettingsV1alpha1(base_api.BaseApiClient):
  """Generated client library for service resourcesettings version v1alpha1."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://resourcesettings.googleapis.com/'
  MTLS_BASE_URL = 'https://resourcesettings.mtls.googleapis.com/'

  _PACKAGE = 'resourcesettings'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1alpha1'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'ResourcesettingsV1alpha1'
  _URL_VERSION = 'v1alpha1'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new resourcesettings handle."""
    url = url or self.BASE_URL
    super(ResourcesettingsV1alpha1, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.folders_settings_value = self.FoldersSettingsValueService(self)
    self.folders_settings = self.FoldersSettingsService(self)
    self.folders = self.FoldersService(self)
    self.organizations_settings_value = self.OrganizationsSettingsValueService(self)
    self.organizations_settings = self.OrganizationsSettingsService(self)
    self.organizations = self.OrganizationsService(self)
    self.projects_settings_value = self.ProjectsSettingsValueService(self)
    self.projects_settings = self.ProjectsSettingsService(self)
    self.projects = self.ProjectsService(self)

  class FoldersSettingsValueService(base_api.BaseApiService):
    """Service class for the folders_settings_value resource."""

    _NAME = 'folders_settings_value'

    def __init__(self, client):
      super(ResourcesettingsV1alpha1.FoldersSettingsValueService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a setting value. Returns a `google.rpc.Status` with `google.rpc.Code.NOT_FOUND` if the setting does not exist. Returns a `google.rpc.Status` with `google.rpc.Code.ALREADY_EXISTS` if the setting value already exists on the given Cloud resource. Returns a `google.rpc.Status` with `google.rpc.Code.FAILED_PRECONDITION` if the setting is flagged as read only.

      Args:
        request: (ResourcesettingsFoldersSettingsValueCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudResourcesettingsV1alpha1SettingValue) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/folders/{foldersId}/settings/{settingsId}/value',
        http_method='POST',
        method_id='resourcesettings.folders.settings.value.create',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='googleCloudResourcesettingsV1alpha1SettingValue',
        request_type_name='ResourcesettingsFoldersSettingsValueCreateRequest',
        response_type_name='GoogleCloudResourcesettingsV1alpha1SettingValue',
        supports_download=False,
    )

  class FoldersSettingsService(base_api.BaseApiService):
    """Service class for the folders_settings resource."""

    _NAME = 'folders_settings'

    def __init__(self, client):
      super(ResourcesettingsV1alpha1.FoldersSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def DeleteValue(self, request, global_params=None):
      r"""Deletes a setting value. If the setting value does not exist, the operation is a no-op. Returns a `google.rpc.Status` with `google.rpc.Code.NOT_FOUND` if the setting or the setting value does not exist. The setting value will not exist if a prior call to `DeleteSetting` for the setting value already returned a success code. Returns a `google.rpc.Status` with `google.rpc.Code.FAILED_PRECONDITION` if the setting is flagged as read only.

      Args:
        request: (ResourcesettingsFoldersSettingsDeleteValueRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleProtobufEmpty) The response message.
      """
      config = self.GetMethodConfig('DeleteValue')
      return self._RunMethod(
          config, request, global_params=global_params)

    DeleteValue.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/folders/{foldersId}/settings/{settingsId}/value',
        http_method='DELETE',
        method_id='resourcesettings.folders.settings.deleteValue',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['ignoreReadOnly'],
        relative_path='v1alpha1/{+name}',
        request_field='',
        request_type_name='ResourcesettingsFoldersSettingsDeleteValueRequest',
        response_type_name='GoogleProtobufEmpty',
        supports_download=False,
    )

    def GetValue(self, request, global_params=None):
      r"""Gets a setting value. Returns a `google.rpc.Status` with `google.rpc.Code.NOT_FOUND` if the setting value does not exist.

      Args:
        request: (ResourcesettingsFoldersSettingsGetValueRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudResourcesettingsV1alpha1SettingValue) The response message.
      """
      config = self.GetMethodConfig('GetValue')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetValue.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/folders/{foldersId}/settings/{settingsId}/value',
        http_method='GET',
        method_id='resourcesettings.folders.settings.getValue',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='',
        request_type_name='ResourcesettingsFoldersSettingsGetValueRequest',
        response_type_name='GoogleCloudResourcesettingsV1alpha1SettingValue',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all the settings that are available on the Cloud resource `parent`.

      Args:
        request: (ResourcesettingsFoldersSettingsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudResourcesettingsV1alpha1ListSettingsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/folders/{foldersId}/settings',
        http_method='GET',
        method_id='resourcesettings.folders.settings.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1alpha1/{+parent}/settings',
        request_field='',
        request_type_name='ResourcesettingsFoldersSettingsListRequest',
        response_type_name='GoogleCloudResourcesettingsV1alpha1ListSettingsResponse',
        supports_download=False,
    )

    def LookupEffectiveValue(self, request, global_params=None):
      r"""Computes the effective setting value of a setting at the Cloud resource `parent`. The effective setting value is the calculated setting value at a Cloud resource and evaluates to one of the following options in the given order (the next option is used if the previous one does not exist): 1. the setting value on the given resource 2. the setting value on the given resource's nearest ancestor 3. the setting's default value 4. an empty setting value, defined as a `SettingValue` with all fields unset Returns a `google.rpc.Status` with `google.rpc.Code.NOT_FOUND` if the setting does not exist.

      Args:
        request: (ResourcesettingsFoldersSettingsLookupEffectiveValueRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudResourcesettingsV1alpha1SettingValue) The response message.
      """
      config = self.GetMethodConfig('LookupEffectiveValue')
      return self._RunMethod(
          config, request, global_params=global_params)

    LookupEffectiveValue.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/folders/{foldersId}/settings/{settingsId}:lookupEffectiveValue',
        http_method='GET',
        method_id='resourcesettings.folders.settings.lookupEffectiveValue',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v1alpha1/{+parent}:lookupEffectiveValue',
        request_field='',
        request_type_name='ResourcesettingsFoldersSettingsLookupEffectiveValueRequest',
        response_type_name='GoogleCloudResourcesettingsV1alpha1SettingValue',
        supports_download=False,
    )

    def Search(self, request, global_params=None):
      r"""Searches for all setting values that exist on the resource `parent`. The setting values are not limited to those of a particular setting.

      Args:
        request: (ResourcesettingsFoldersSettingsSearchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudResourcesettingsV1alpha1SearchSettingValuesResponse) The response message.
      """
      config = self.GetMethodConfig('Search')
      return self._RunMethod(
          config, request, global_params=global_params)

    Search.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/folders/{foldersId}/settings:search',
        http_method='GET',
        method_id='resourcesettings.folders.settings.search',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1alpha1/{+parent}/settings:search',
        request_field='',
        request_type_name='ResourcesettingsFoldersSettingsSearchRequest',
        response_type_name='GoogleCloudResourcesettingsV1alpha1SearchSettingValuesResponse',
        supports_download=False,
    )

    def UpdateValue(self, request, global_params=None):
      r"""Updates a setting value. Returns a `google.rpc.Status` with `google.rpc.Code.NOT_FOUND` if the setting or the setting value does not exist. Returns a `google.rpc.Status` with `google.rpc.Code.FAILED_PRECONDITION` if the setting is flagged as read only. Returns a `google.rpc.Status` with `google.rpc.Code.ABORTED` if the etag supplied in the request does not match the persisted etag of the setting value. Note: the supplied setting value will perform a full overwrite of all fields.

      Args:
        request: (ResourcesettingsFoldersSettingsUpdateValueRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudResourcesettingsV1alpha1SettingValue) The response message.
      """
      config = self.GetMethodConfig('UpdateValue')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateValue.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/folders/{foldersId}/settings/{settingsId}/value',
        http_method='PATCH',
        method_id='resourcesettings.folders.settings.updateValue',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='googleCloudResourcesettingsV1alpha1SettingValue',
        request_type_name='ResourcesettingsFoldersSettingsUpdateValueRequest',
        response_type_name='GoogleCloudResourcesettingsV1alpha1SettingValue',
        supports_download=False,
    )

  class FoldersService(base_api.BaseApiService):
    """Service class for the folders resource."""

    _NAME = 'folders'

    def __init__(self, client):
      super(ResourcesettingsV1alpha1.FoldersService, self).__init__(client)
      self._upload_configs = {
          }

  class OrganizationsSettingsValueService(base_api.BaseApiService):
    """Service class for the organizations_settings_value resource."""

    _NAME = 'organizations_settings_value'

    def __init__(self, client):
      super(ResourcesettingsV1alpha1.OrganizationsSettingsValueService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a setting value. Returns a `google.rpc.Status` with `google.rpc.Code.NOT_FOUND` if the setting does not exist. Returns a `google.rpc.Status` with `google.rpc.Code.ALREADY_EXISTS` if the setting value already exists on the given Cloud resource. Returns a `google.rpc.Status` with `google.rpc.Code.FAILED_PRECONDITION` if the setting is flagged as read only.

      Args:
        request: (ResourcesettingsOrganizationsSettingsValueCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudResourcesettingsV1alpha1SettingValue) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/organizations/{organizationsId}/settings/{settingsId}/value',
        http_method='POST',
        method_id='resourcesettings.organizations.settings.value.create',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='googleCloudResourcesettingsV1alpha1SettingValue',
        request_type_name='ResourcesettingsOrganizationsSettingsValueCreateRequest',
        response_type_name='GoogleCloudResourcesettingsV1alpha1SettingValue',
        supports_download=False,
    )

  class OrganizationsSettingsService(base_api.BaseApiService):
    """Service class for the organizations_settings resource."""

    _NAME = 'organizations_settings'

    def __init__(self, client):
      super(ResourcesettingsV1alpha1.OrganizationsSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def DeleteValue(self, request, global_params=None):
      r"""Deletes a setting value. If the setting value does not exist, the operation is a no-op. Returns a `google.rpc.Status` with `google.rpc.Code.NOT_FOUND` if the setting or the setting value does not exist. The setting value will not exist if a prior call to `DeleteSetting` for the setting value already returned a success code. Returns a `google.rpc.Status` with `google.rpc.Code.FAILED_PRECONDITION` if the setting is flagged as read only.

      Args:
        request: (ResourcesettingsOrganizationsSettingsDeleteValueRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleProtobufEmpty) The response message.
      """
      config = self.GetMethodConfig('DeleteValue')
      return self._RunMethod(
          config, request, global_params=global_params)

    DeleteValue.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/organizations/{organizationsId}/settings/{settingsId}/value',
        http_method='DELETE',
        method_id='resourcesettings.organizations.settings.deleteValue',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['ignoreReadOnly'],
        relative_path='v1alpha1/{+name}',
        request_field='',
        request_type_name='ResourcesettingsOrganizationsSettingsDeleteValueRequest',
        response_type_name='GoogleProtobufEmpty',
        supports_download=False,
    )

    def GetValue(self, request, global_params=None):
      r"""Gets a setting value. Returns a `google.rpc.Status` with `google.rpc.Code.NOT_FOUND` if the setting value does not exist.

      Args:
        request: (ResourcesettingsOrganizationsSettingsGetValueRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudResourcesettingsV1alpha1SettingValue) The response message.
      """
      config = self.GetMethodConfig('GetValue')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetValue.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/organizations/{organizationsId}/settings/{settingsId}/value',
        http_method='GET',
        method_id='resourcesettings.organizations.settings.getValue',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='',
        request_type_name='ResourcesettingsOrganizationsSettingsGetValueRequest',
        response_type_name='GoogleCloudResourcesettingsV1alpha1SettingValue',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all the settings that are available on the Cloud resource `parent`.

      Args:
        request: (ResourcesettingsOrganizationsSettingsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudResourcesettingsV1alpha1ListSettingsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/organizations/{organizationsId}/settings',
        http_method='GET',
        method_id='resourcesettings.organizations.settings.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1alpha1/{+parent}/settings',
        request_field='',
        request_type_name='ResourcesettingsOrganizationsSettingsListRequest',
        response_type_name='GoogleCloudResourcesettingsV1alpha1ListSettingsResponse',
        supports_download=False,
    )

    def LookupEffectiveValue(self, request, global_params=None):
      r"""Computes the effective setting value of a setting at the Cloud resource `parent`. The effective setting value is the calculated setting value at a Cloud resource and evaluates to one of the following options in the given order (the next option is used if the previous one does not exist): 1. the setting value on the given resource 2. the setting value on the given resource's nearest ancestor 3. the setting's default value 4. an empty setting value, defined as a `SettingValue` with all fields unset Returns a `google.rpc.Status` with `google.rpc.Code.NOT_FOUND` if the setting does not exist.

      Args:
        request: (ResourcesettingsOrganizationsSettingsLookupEffectiveValueRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudResourcesettingsV1alpha1SettingValue) The response message.
      """
      config = self.GetMethodConfig('LookupEffectiveValue')
      return self._RunMethod(
          config, request, global_params=global_params)

    LookupEffectiveValue.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/organizations/{organizationsId}/settings/{settingsId}:lookupEffectiveValue',
        http_method='GET',
        method_id='resourcesettings.organizations.settings.lookupEffectiveValue',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v1alpha1/{+parent}:lookupEffectiveValue',
        request_field='',
        request_type_name='ResourcesettingsOrganizationsSettingsLookupEffectiveValueRequest',
        response_type_name='GoogleCloudResourcesettingsV1alpha1SettingValue',
        supports_download=False,
    )

    def Search(self, request, global_params=None):
      r"""Searches for all setting values that exist on the resource `parent`. The setting values are not limited to those of a particular setting.

      Args:
        request: (ResourcesettingsOrganizationsSettingsSearchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudResourcesettingsV1alpha1SearchSettingValuesResponse) The response message.
      """
      config = self.GetMethodConfig('Search')
      return self._RunMethod(
          config, request, global_params=global_params)

    Search.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/organizations/{organizationsId}/settings:search',
        http_method='GET',
        method_id='resourcesettings.organizations.settings.search',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1alpha1/{+parent}/settings:search',
        request_field='',
        request_type_name='ResourcesettingsOrganizationsSettingsSearchRequest',
        response_type_name='GoogleCloudResourcesettingsV1alpha1SearchSettingValuesResponse',
        supports_download=False,
    )

    def UpdateValue(self, request, global_params=None):
      r"""Updates a setting value. Returns a `google.rpc.Status` with `google.rpc.Code.NOT_FOUND` if the setting or the setting value does not exist. Returns a `google.rpc.Status` with `google.rpc.Code.FAILED_PRECONDITION` if the setting is flagged as read only. Returns a `google.rpc.Status` with `google.rpc.Code.ABORTED` if the etag supplied in the request does not match the persisted etag of the setting value. Note: the supplied setting value will perform a full overwrite of all fields.

      Args:
        request: (ResourcesettingsOrganizationsSettingsUpdateValueRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudResourcesettingsV1alpha1SettingValue) The response message.
      """
      config = self.GetMethodConfig('UpdateValue')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateValue.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/organizations/{organizationsId}/settings/{settingsId}/value',
        http_method='PATCH',
        method_id='resourcesettings.organizations.settings.updateValue',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='googleCloudResourcesettingsV1alpha1SettingValue',
        request_type_name='ResourcesettingsOrganizationsSettingsUpdateValueRequest',
        response_type_name='GoogleCloudResourcesettingsV1alpha1SettingValue',
        supports_download=False,
    )

  class OrganizationsService(base_api.BaseApiService):
    """Service class for the organizations resource."""

    _NAME = 'organizations'

    def __init__(self, client):
      super(ResourcesettingsV1alpha1.OrganizationsService, self).__init__(client)
      self._upload_configs = {
          }

  class ProjectsSettingsValueService(base_api.BaseApiService):
    """Service class for the projects_settings_value resource."""

    _NAME = 'projects_settings_value'

    def __init__(self, client):
      super(ResourcesettingsV1alpha1.ProjectsSettingsValueService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a setting value. Returns a `google.rpc.Status` with `google.rpc.Code.NOT_FOUND` if the setting does not exist. Returns a `google.rpc.Status` with `google.rpc.Code.ALREADY_EXISTS` if the setting value already exists on the given Cloud resource. Returns a `google.rpc.Status` with `google.rpc.Code.FAILED_PRECONDITION` if the setting is flagged as read only.

      Args:
        request: (ResourcesettingsProjectsSettingsValueCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudResourcesettingsV1alpha1SettingValue) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/settings/{settingsId}/value',
        http_method='POST',
        method_id='resourcesettings.projects.settings.value.create',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='googleCloudResourcesettingsV1alpha1SettingValue',
        request_type_name='ResourcesettingsProjectsSettingsValueCreateRequest',
        response_type_name='GoogleCloudResourcesettingsV1alpha1SettingValue',
        supports_download=False,
    )

  class ProjectsSettingsService(base_api.BaseApiService):
    """Service class for the projects_settings resource."""

    _NAME = 'projects_settings'

    def __init__(self, client):
      super(ResourcesettingsV1alpha1.ProjectsSettingsService, self).__init__(client)
      self._upload_configs = {
          }

    def DeleteValue(self, request, global_params=None):
      r"""Deletes a setting value. If the setting value does not exist, the operation is a no-op. Returns a `google.rpc.Status` with `google.rpc.Code.NOT_FOUND` if the setting or the setting value does not exist. The setting value will not exist if a prior call to `DeleteSetting` for the setting value already returned a success code. Returns a `google.rpc.Status` with `google.rpc.Code.FAILED_PRECONDITION` if the setting is flagged as read only.

      Args:
        request: (ResourcesettingsProjectsSettingsDeleteValueRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleProtobufEmpty) The response message.
      """
      config = self.GetMethodConfig('DeleteValue')
      return self._RunMethod(
          config, request, global_params=global_params)

    DeleteValue.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/settings/{settingsId}/value',
        http_method='DELETE',
        method_id='resourcesettings.projects.settings.deleteValue',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['ignoreReadOnly'],
        relative_path='v1alpha1/{+name}',
        request_field='',
        request_type_name='ResourcesettingsProjectsSettingsDeleteValueRequest',
        response_type_name='GoogleProtobufEmpty',
        supports_download=False,
    )

    def GetValue(self, request, global_params=None):
      r"""Gets a setting value. Returns a `google.rpc.Status` with `google.rpc.Code.NOT_FOUND` if the setting value does not exist.

      Args:
        request: (ResourcesettingsProjectsSettingsGetValueRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudResourcesettingsV1alpha1SettingValue) The response message.
      """
      config = self.GetMethodConfig('GetValue')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetValue.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/settings/{settingsId}/value',
        http_method='GET',
        method_id='resourcesettings.projects.settings.getValue',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='',
        request_type_name='ResourcesettingsProjectsSettingsGetValueRequest',
        response_type_name='GoogleCloudResourcesettingsV1alpha1SettingValue',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all the settings that are available on the Cloud resource `parent`.

      Args:
        request: (ResourcesettingsProjectsSettingsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudResourcesettingsV1alpha1ListSettingsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/settings',
        http_method='GET',
        method_id='resourcesettings.projects.settings.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1alpha1/{+parent}/settings',
        request_field='',
        request_type_name='ResourcesettingsProjectsSettingsListRequest',
        response_type_name='GoogleCloudResourcesettingsV1alpha1ListSettingsResponse',
        supports_download=False,
    )

    def LookupEffectiveValue(self, request, global_params=None):
      r"""Computes the effective setting value of a setting at the Cloud resource `parent`. The effective setting value is the calculated setting value at a Cloud resource and evaluates to one of the following options in the given order (the next option is used if the previous one does not exist): 1. the setting value on the given resource 2. the setting value on the given resource's nearest ancestor 3. the setting's default value 4. an empty setting value, defined as a `SettingValue` with all fields unset Returns a `google.rpc.Status` with `google.rpc.Code.NOT_FOUND` if the setting does not exist.

      Args:
        request: (ResourcesettingsProjectsSettingsLookupEffectiveValueRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudResourcesettingsV1alpha1SettingValue) The response message.
      """
      config = self.GetMethodConfig('LookupEffectiveValue')
      return self._RunMethod(
          config, request, global_params=global_params)

    LookupEffectiveValue.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/settings/{settingsId}:lookupEffectiveValue',
        http_method='GET',
        method_id='resourcesettings.projects.settings.lookupEffectiveValue',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v1alpha1/{+parent}:lookupEffectiveValue',
        request_field='',
        request_type_name='ResourcesettingsProjectsSettingsLookupEffectiveValueRequest',
        response_type_name='GoogleCloudResourcesettingsV1alpha1SettingValue',
        supports_download=False,
    )

    def Search(self, request, global_params=None):
      r"""Searches for all setting values that exist on the resource `parent`. The setting values are not limited to those of a particular setting.

      Args:
        request: (ResourcesettingsProjectsSettingsSearchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudResourcesettingsV1alpha1SearchSettingValuesResponse) The response message.
      """
      config = self.GetMethodConfig('Search')
      return self._RunMethod(
          config, request, global_params=global_params)

    Search.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/settings:search',
        http_method='GET',
        method_id='resourcesettings.projects.settings.search',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1alpha1/{+parent}/settings:search',
        request_field='',
        request_type_name='ResourcesettingsProjectsSettingsSearchRequest',
        response_type_name='GoogleCloudResourcesettingsV1alpha1SearchSettingValuesResponse',
        supports_download=False,
    )

    def UpdateValue(self, request, global_params=None):
      r"""Updates a setting value. Returns a `google.rpc.Status` with `google.rpc.Code.NOT_FOUND` if the setting or the setting value does not exist. Returns a `google.rpc.Status` with `google.rpc.Code.FAILED_PRECONDITION` if the setting is flagged as read only. Returns a `google.rpc.Status` with `google.rpc.Code.ABORTED` if the etag supplied in the request does not match the persisted etag of the setting value. Note: the supplied setting value will perform a full overwrite of all fields.

      Args:
        request: (ResourcesettingsProjectsSettingsUpdateValueRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudResourcesettingsV1alpha1SettingValue) The response message.
      """
      config = self.GetMethodConfig('UpdateValue')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateValue.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/settings/{settingsId}/value',
        http_method='PATCH',
        method_id='resourcesettings.projects.settings.updateValue',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='googleCloudResourcesettingsV1alpha1SettingValue',
        request_type_name='ResourcesettingsProjectsSettingsUpdateValueRequest',
        response_type_name='GoogleCloudResourcesettingsV1alpha1SettingValue',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(ResourcesettingsV1alpha1.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



#!/usr/bin/env python
"""Client options class.

Client options provide a consistent interface for user options to be defined
across clients.

You can pass a client options object to a client.

.. code-block:: python

    from google.api_core.client_options import ClientOptions
    from google.cloud.vision_v1 import ImageAnnotatorClient

    def get_client_cert():
        # code to load client certificate and private key.
        return client_cert_bytes, client_private_key_bytes

    options = ClientOptions(api_endpoint="foo.googleapis.com",
        client_cert_source=get_client_cert)

    client = ImageAnnotatorClient(client_options=options)

You can also pass a mapping object.

.. code-block:: python

    from google.cloud.vision_v1 import ImageAnnotatorClient

    client = ImageAnnotatorClient(
        client_options={
            "api_endpoint": "foo.googleapis.com",
            "client_cert_source" : get_client_cert
        })


"""


class ClientOptions(object):
    """Client Options used to set options on clients.

    Args:
        api_endpoint (Optional[str]): The desired API endpoint, e.g.,
            compute.googleapis.com
        client_cert_source (Optional[Callable[[], (bytes, bytes)]]): A callback
            which returns client certificate bytes and private key bytes both in
            PEM format. ``client_cert_source`` and ``client_encrypted_cert_source``
            are mutually exclusive.
        client_encrypted_cert_source (Optional[Callable[[], (str, str, bytes)]]):
            A callback which returns client certificate file path, encrypted
            private key file path, and the passphrase bytes.``client_cert_source``
            and ``client_encrypted_cert_source`` are mutually exclusive.
        quota_project_id (Optional[str]): A project name that a client's
            quota belongs to.
        credentials_file (Optional[str]): A path to a file storing credentials.
            ``credentials_file` and ``api_key`` are mutually exclusive.
        scopes (Optional[Sequence[str]]): OAuth access token override scopes.
        api_key (Optional[str]): Google API key. ``credentials_file`` and
            ``api_key`` are mutually exclusive.
        api_audience (Optional[str]): The intended audience for the API calls
            to the service that will be set when using certain 3rd party
            authentication flows. Audience is typically a resource identifier.
            If not set, the service endpoint value will be used as a default.
            An example of a valid ``api_audience`` is: "https://language.googleapis.com".

    Raises:
        ValueError: If both ``client_cert_source`` and ``client_encrypted_cert_source``
            are provided, or both ``credentials_file`` and ``api_key`` are provided.
    """

    def __init__(
        self,
        api_endpoint=None,
        client_cert_source=None,
        client_encrypted_cert_source=None,
        quota_project_id=None,
        credentials_file=None,
        scopes=None,
        api_key=None,
        api_audience=None,
    ):
        if client_cert_source and client_encrypted_cert_source:
            raise ValueError(
                "client_cert_source and client_encrypted_cert_source are mutually exclusive"
            )
        if api_key and credentials_file:
            raise ValueError("api_key and credentials_file are mutually exclusive")
        self.api_endpoint = api_endpoint
        self.client_cert_source = client_cert_source
        self.client_encrypted_cert_source = client_encrypted_cert_source
        self.quota_project_id = quota_project_id
        self.credentials_file = credentials_file
        self.scopes = scopes
        self.api_key = api_key
        self.api_audience = api_audience

    def __repr__(self):
        return "ClientOptions: " + repr(self.__dict__)


def from_dict(options):
    """Construct a client options object from a mapping object.

    Args:
        options (collections.abc.Mapping): A mapping object with client options.
            See the docstring for ClientOptions for details on valid arguments.
    """

    client_options = ClientOptions()

    for key, value in options.items():
        if hasattr(client_options, key):
            setattr(client_options, key, value)
        else:
            raise ValueError("ClientOptions does not accept an option '" + key + "'")

    return client_options
